<?php
/*
 *	Top level script
 *
 *	Requires working TNG installations (including email)
 *		from 12.1 up to current v13.
 * Sets session vars with user privileges
 * Uses the tng current language if available in fgw; otherwise English
 *
 * No special accommodations for CMS as there are too many variations
 * of them and maintaining an FGW integration is too time consumeing.
 *
*/

/***************************************************************
	VARIABLE INITIALIZATIONS
***************************************************************/	
$id = '';
$fkey = '';
$persfamID = '';
$max_loglines = 1000;
$gedcom = '';
$familyID = '';
$personID = '';
$husbID = '';
$wifeID = '';

/***************************************************************
	LOAD LIBRARIES
***************************************************************/
/*
 * Bypass the TNG checklogin file called in tng_begin.php until we analyze
 * user access status; otherwise guest with id and fkey would not pass
 */
$nologin=1;
require_once 'tng_begin.php'; // no login check

// Load user configs and FGW language support
require_once 'fgw_inits.php';

// generalized database interface
require_once 'fgw_dbconn.php';

require_once 'fgw_lib.php';
require_once 'fgw_logger.php';	// FGW visitor activity log
require_once 'fgw_version.php';

$dateformat = $tngconfig['preferEuro'] == 'true' ? "dd/mm/yyyy" : "mm/dd/yyyy";
//echo __LINE__,'<pre>';print_r($dateformat);exit;
// SEE IF USER NEEDS TO LOGIN TO TNG TO VIEW THIS PAGE

/***************************************************************
	AUTHORIZE ACCESS
***************************************************************/
while( true) 
{
	// bypass login if user has id and fkey provided by site authority 
	// to view worksheet
	if( !empty( $_GET['id'] ) && !empty( $_GET['fkey'] ) ) break;

	// everybody else needs to be authorized per user config and
	// get working values in session vars	
	unset( $nologin );
	require_once 'checklogin.php';

	// bypass login if member is logged in
	if( !empty($_SESSION['logged_in'] ) ) break;

	// bypass login if both site and fgw are unrestricted
	if( empty( $requirelogin ) && empty( $options['restricted'] ) ) break;

	// login required here
	$_SESSION['destinationpage8'] = $_SERVER['REQUEST_URI'];
	header("Location: {$cms['tngpath']}login.php" );
	exit;
}

// IDENTIFY SESSION SUPER USERS
$tngrole = isset($_SESSION['tngrole']) ? $_SESSION['tngrole'] : '';
$editors = !empty( $options['editors'] )  ? 1 : 0;
if( $tngrole == 'admin' || ( $editors && $tngrole == 'editor' ) ) 
{
	$_SESSION['fgw_superuser'] = 1;
}
else {
	$_SESSION['fgw_superuser'] = 0;
}

/* Check that a worksheet db table has been created */
if( empty( $options['table'] ) || !db_table_exists( $options['table'] ) ) 
{
	if( $tngrole == 'admin' ) 
	{
		header('Location:fgw_setup.php');
		exit;
	} 
	else 
	{
		trigger_error( $admtext['ws_no_table'], E_USER_ERROR );
		exit;	
	}
   die();
}

/*
 * Only log FGW visitors (as visitors) once.  If they perform other functions
 * while they are here, such as submitting, editing or deleting a worksheet
 * their actions will be noted each time in the visitor log.
 */
if( empty( $_SESSION['fgw_opened'] ) ) 
{
	$_SESSION['fgw_opened'] = 1;

	require_once 'fgw_logger.php';

	// RECORD THIS ACCESS IN VISITOR LOG
	$logstring = "FGW:ws_accessedby:";
	fgw_writelog( $options['userlog'], $logstring,
				$options['max_loglines'], $options['nologip'] );
}

/***************************************************************
	PROCESS INCOMING GET DATA
	processvars.php has already broken these out and cleaned them
***************************************************************/	
if( !empty( $_GET ) && is_array( $_GET ) ) 
{ 
	if( !empty($new) ) 
	{
		//$_SESSION['fgw_page8'] = $tngdomain.'/'.$fgw_path.'fgw.php';
		$_SESSION['fgw_id'] = '';
		$_SESSION['fgw_fkey'] = '';
	}
	else 
	{
		//$_SESSION['fgw_page8'] = $tngdomain.'/'.$fgw_path.'fgw.php';
		$qstring = $_SERVER['QUERY_STRING'];
		if( !empty( $qstring ) ) 
		{
			//echo __LINE__,'  ';
			//$_SESSION['fgw_page8'] .= '?'.$qstring;
			if( !empty( $id ) ) $_SESSION['fgw_id'] = $id;
			if( !empty( $fkey ) ) $_SESSION['fgw_fkey'] = $fkey;
		}
	}
}
else 
{
	if( !empty( $_SESSION['fgw_id'] ) ) 
	{
		$id = $_SESSION['fgw_id'];
	}
	if( !empty( $_SESSION['fgw_fkey'] ) ) 
	{
		$fkey = $_SESSION['fgw_fkey'];
	}
}

// fgw_superuser has been set in fgw.php
$is_superuser = !empty( $_SESSION['fgw_superuser'] ) ? 1 : 0;
$currentuser = !empty( $_SESSION['currentuser'] ) ? $_SESSION['currentuser'] : '';
$allow_lds = !empty($_SESSION['allow_lds']) ? $_SESSION['allow_lds'] : '';

/***************************************************************
	MAINTAIN FGW FORMS DATA TABLE
***************************************************************/	

// HAS WORKSHEET TABLE BEEN UPDATED FROM VERY OLD VERSION
if( !db_column_exists( $options['table'], 'gedcom' ) ) 
{
	trigger_error( $admtext['ws_no_upgrade'], E_USER_ERROR );
   die();
 }

if( !isset( $_SESSION['fgw_guest'] ) ) $_SESSION['fgw_guest'] = 0;

/**************************************************************
	WORKSHEET ADMIN AREA OPS: DELETE, PREV, NEXT
***************************************************************/
if( !empty( $_GET['del'] ) ) 
{
   if( !empty( $_GET['formID'] ) ) 
   {
	   	if( $is_superuser ) 
		{
	   		fgw_delete_worksheet( $_GET['formID'] );
			// update user activity log
		  	$logstring = "fgw#$formID:ws_deletedby:";
		  	fgw_writelog( $options['userlog'], $logstring,
		  		$options['max_loglines'], $options['nologip'] );
			header("Location: fgw_search.php");
			exit;
		}
		else 
		{
			trigger_error( $admtext['wse_hack'], E_USER_ERROR );
   		die();
		}
   }
}
// from $_POST
elseif( !empty( $prev) || !empty( $next ) ) 
{
	if( $newform = fgw_nav2( $_POST ) ) 
	{
	}
	header("Location: fgw.php?id=$newform");
	exit;
}

// TAILORED WORKSHEET INTRODUCTION FOR ADMINS
if( isset($_SESSION['tngrole']) && $_SESSION['tngrole'] == 'admin' ) 
{
	$text['ws_intro'] = $text['ws_admin_intro'];
}

// Default instruction text
$notify = "<span class='info'><i>{$text['ws_intro']}</i></span>";

// default asterisk for required fields
define( "STAR", "<span style='color:{$options['error_msg_color']}; font-size:180%;'><strong>*</strong></span>");

/***************************************************************
	SET UP ID AND TREE DATA FROM FORM 
***************************************************************/
// FAMILY OR INDIVIDUAL ON FORM
if( !empty( $persfamID ) ) {
   if( strtoupper( $persfamID[0] == $options['fid_prefix'] ) )
   {
      $familyID = $persfamID;
   }
   elseif( strtoupper( $persfamID[0] == $options['pid_prefix'] ) )
   {
      $personID = $persfamID;
   }
}
else 
{
   $persfamID = '';
}

if( !empty( $tree ) )
   $gedcom = $tree;

// array of permitted trees (gedcom)
$trees = fgw_fetch_permitted_trees( $options['treelist'] );

if( empty( $gedcom ) ) $gedcom = $trees[0]['gedcom'];

$data = array();
$disabled_tree = false;
/***************************************************************
	ASSEMBLE FORM DATA PER DIFFERENT KINDS OF REQUEST
***************************************************************/
// REQUESTING DATABASED WORKSHEET FROM LINK OR LIST
if( $id ) 
{
	while( true ) 
	{ // validate user privileges; break to display form

		unset( $_SESSION['fgw_newform']);

		$valid_id = $valid_fkey = $valid_tree = $closed = false;
		if( is_numeric( $id ) ) 
		{
			$form = fgw_fetch_form_record( $id );
			if( !empty( $form['formID'] ) ) 
			{
				if( empty( $form['gedcom'] ) ) 
				{
					trigger_error( $admtext['wse_no_gedcom'], E_USER_ERROR );
				   die();
				}
				$valid_id = true;
				$valid_fkey = $fkey == $form['fkey'];
				$valid_tree = fgw_in_trees( $form['gedcom'], $trees );
				$closed = $form['closed'];
			}
		}

		if( !$valid_id ) 
		{
			if( $is_superuser ) 
			{
				header("Location: fgw_search.php");
				exit;
			}
			else 
			{ // display public error and logit
				trigger_error( $admtext['wse_no_worksheet'], E_USER_ERROR );
				die();
			}
		}

		// even superuser must have access privilege to the requested tree
		if( $is_superuser ) 
		{
			if( !$valid_tree ) 
			{
				trigger_error( $admtext['wse_no_auth'], E_USER_ERROR );
				die();
			}
			else 
			{
				break;
			}
		}

		// submitter must have valid key and form must be open
		if( $valid_fkey ) 
		{
			if( !$closed ) 
			{
				break;
			}
			else 
			{
				require_once $fgw_language_file;
				echo '<h2>',__LINE__,': ',$text['ws_closed_info'],'</h2>';
				die();
			}
		}
		else 
		{
				trigger_error( $admtext['wse_badkey'].' (id='.$id.')', E_USER_ERROR );
				die();
		}
		break;
	} // while (processing)

	/*
     * EXTRACT FORM DATA
     * Form data is stored in the data base as a coded 'blob'
     * which must be decoded and unserialized into discrete data items.
     */
	$data = unserialize( base64_decode($form['data'] ) );
	//echo __LINE__,'<pre>';print_r($data);exit;


	// OLDER DATA RECORDS WILL THROW WARNINGS WITHOUT THE FOLLOWING
	// These fields were not in the very early records for FGW (@ 2014)
	// Saving these records as admin will eliminate the warnings at the source
	if( !isset( $data['marr']['husband'] ) ) 
	{
		$data['marr']['husband'] = '';
	}
	if( !isset( $data['marr']['wife'] ) ) 
	{
		$data['marr']['wife'] = '';
	}

	if( !isset( $data['husb']['title'] ) ) 
	{ 	//wrong -> parents
		$data['husb']['title'] = '';
	}

	if( !isset( $data['husb']['mother']['title'] ) ) 
	{ 	//wrong -> parents
		$data['husb']['mother']['title'] = '';
	}

	if( !isset( $data['husb']['father']['title'] ) ) 
	{  	// wrong -> parents
		$data['husb']['father']['title'] = '';
	}

	if( !isset( $data['husb']['parsfam']['sealdate'] ) )
	{	//wrong -> parents
		$data['husb']['parsfam']['sealdate'] = '';
	}

	if( !isset( $data['husb']['parsfam']['sealplace'] ) ) 
	{
		$data['husb']['parsfam']['sealplace'] = '';
	}

	if( !isset( $data['wife']['title'] ) ) 
	{	//wrong -> parents
		$data['wife']['title'] = '';
	}

	if( !isset( $data['wife']['mother']['title'] ) ) 
	{	//wrong -> parents
		$data['wife']['mother']['title'] = '';
	}

	if( !isset( $data['wife']['father']['title'] ) ) 
	{	// wrong -> parents
		$data['wife']['father']['title'] = '';
	}

	if( !isset( $data['wife']['parsfam']['sealdate'] ) )
	{	//wrong -> parents
		$data['wife']['parsfam']['sealdate'] = '';
	}

	if( !isset( $data['wife']['parsfam']['sealplace'] ) ) 
	{
		$data['wife']['parsfam']['sealplace'] = '';
	}

	// QUASH NOTICES
	if( !isset( $data['husb']['father']['gedcom'] )) 
	{
		$data['husb']['father']['gedcom'] = isset($gedcom) ? $gedcom : '';
	}

	if( !isset( $data['husb']['mother']['gedcom'] )) 
	{
		$data['husb']['mother']['gedcom'] = isset($gedcom) ? $gedcom : '';
	}

	if( !isset( $data['wife']['father']['gedcom'] )) 
	{
		$data['wife']['father']['gedcom'] = isset($gedcom) ? $gedcom : '';
	}

	if( !isset( $data['wife']['mother']['gedcom'] )) 
	{
		$data['wife']['mother']['gedcom'] = isset($gedcom) ? $gedcom : '';
	}

	if( !isset( $data['husb']['father']['PersonID'] )) 
	{
		$data['husb']['father']['personID'] = '';
	}

	if( !isset( $data['husb']['mother']['PersonID'] )) 
	{
		$data['husb']['mother']['personID'] = '';
	}

	if( !isset( $data['wife']['father']['personID'] )) 
	{
		$data['wife']['father']['personID'] = '';
	}

	if( !isset( $data['wife']['mother']['personID'] )) 
	{
		$data['wife']['mother']['personID'] = '';
	}

   /*
    * RETRIEVE METADATA FROM FORM RECORD
    */
	foreach( $form as $key => $value ) 
	{
		if( $key == 'data' ) continue;
		${$key} = $value;
	}

	// disable tree selection for invited guest
	if( !$is_superuser && empty( $currentuser ) ) 
	{
		$disabled_tree = true;
	}
}
else 
{
	$_SESSION['fgw_newform'] = 1;
}

// REQUESTING A CHILD-DERIVED FORM (NEW PERSON)
if( empty( $data ) && !empty( $sv ) && !empty( $_SESSION['fgw_spouse'] ) ) 
{
    $spouse = $_SESSION['fgw_spouse'];
    //print_r( $spouse );exit;
    $role = empty( $spouse['husb'] ) ? 'wife' : 'husb';
    if( empty( $spouse[$role]['personID'] ) ) 
	{
       unset( $_SESSION['fgw_spouse'] );
       unset( $sv );
       //$gedcom = $spouse[$role]['gedcom'];
       $user = $spouse['user'];
       $data = fgw_fetch_persfam_data_empty( $gedcom );

       foreach( $spouse[$role] as $key => $val ) 
	   {
          // echo $key,'=>',$val,'<br />';
           $data[$role][$key] = $val;
       }
    }
    else 
	{
      //$gedcom = $spouse['gedcom'];
      $persfamID = $spouse[$role]['personID'];
    }
}

// REQUESTING PRE-FILLED FORM
if( empty( $data ) && !empty( $gedcom ) && !empty( $persfamID ) )
{
   if( $is_superuser || $currentuser || !$options['restricted'] ) 
   {
      $data = fgw_fetch_persfam_data( $gedcom, $persfamID );
      $user = fgw_fetch_user_datapack( $currentuser );
      if( !fgw_in_trees( $gedcom, $trees ) ) 
	  {
			trigger_error( $admtext['wse_no_auth'], E_USER_ERROR );
			die();
      }
   }
   else 
   {  // not logged in -- this is redundant since user is sent to login
		trigger_error( $admtext['wse_no_auth'], E_USER_ERROR );
		die();
  	}
}

// MERGE DATA FROM PARENT FORM FOR NEW CHILD FORM
if( !empty( $data ) && !empty( $sv ) && !empty( $spouse ) ) 
{
   unset( $_SESSION['fgw_spouse'] );
   foreach( $spouse[$role] as $key => $value ) 
   {
      $data[$role][$key] = $value;
   }
   unset( $spouse );
}

// REQUESTING FORM FOR A DIFFERENT TREE (with blank form)
if( !empty( $gedcom ) && empty( $persfamID ) )
{
	if( !fgw_in_trees( $gedcom, $trees ) ) 
	{
		trigger_error( $admtext['wse_no_auth'], E_USER_ERROR );
		die();
	}
}

// REQUESTING A BLANK FORM -- NEED GEDCOM HERE
if( empty( $data ) )  
{
	// ADMIN PERSONAL DATA NOT PRE-FILLED
	if( $is_superuser || $currentuser || empty( $options['restricted'] ) ) 
	{
		if( empty( $gedcom ) ) 
		{
			$gedcom = $trees[0]['gedcom'];
		}
		$data = fgw_fetch_persfam_data_empty( $gedcom );

		if( $is_superuser ) 
		{
			$user = fgw_fetch_user_datapack_empty();
		}
		elseif( !empty( $_SESSION['fgw_user'] ) ) 
		{
			$user = $_SESSION['fgw_user'];
		}
		elseif( $currentuser ) 
		{
				$user = fgw_fetch_user_datapack( $currentuser );
				$_SESSION['fgw_user'] = $user;
		}
		else 
		{ // not logged in - no site restrictions
			$user = fgw_fetch_user_datapack_empty();
		}
	}
	else
	{
		trigger_error( $admtext['wse_no_auth'], E_USER_ERROR );
		die();
	}
}

$tree_selector = fgw_tree_selector( $trees, $gedcom, $disabled_tree );

foreach( $data as $key => $value ) 
{
    ${$key} = $value;
}

// UPDATED TO ALLOW ADMIN CREATE/EDIT - added in v43
$formTitle = empty( $description ) ? fgw_title($husb, $wife) : $description;
//$formTitle = fgw_title( $husb, $wife );

$formTitle = htmlspecialchars_decode( $formTitle, ENT_QUOTES | ENT_HTML5 );

if( !empty($options['show_captcha'] ) ) 
{
   $_SESSION['captcha_code'] = strtolower( substr( fgw_password(8), 4) );
   $fgw_hash = md5( $_SESSION['captcha_code'] );
}

// if background image name is empty string,
// form will not set destinationpage8 value!
$form_bgimg = !empty( $form_bgimg ) ? $form_bgimg : "noimage";


/***************************************************************
	PREPARE AND PUBLISH THE TNG HTML PAGE HEADER
***************************************************************/
/* Publish the TNG html page-header to the browser - no visible content yet. */
$title = $text['ws_fgw'];
$flags['noheader'] = false;
$flags['nobody'] = false;
$flags['noicons'] = false;
if(!isset($flags['scripting'])) $flags['scripting'] = '';

/* User configurable worksheet styling in HTML header */
$flags['scripting'] .= "<link rel='stylesheet' href='css/fgw_style.css?v=13.0' type='text/css' />
<!-- FGW DEFAULT STYLING -->
<style type='text/css'>
<!--
.fgw_container {
	background-color:#fffff9;
}
.fgw_container input[type='text'],
.fgw_container select,
.fgw_container textarea,
.fgw_container .formcell {
	background-color:white;
	font-weight:bold;
}
.fgw_container .formcell .label {
	color:#333;
	background-color:#ddd;
	border-bottom:1px solid #aaa;
}
.fgw_container .warning {color:red;}
#husbdata { border-top:0px solid #aaa;}
#wifedata { border-top:0px solid #aaa;}
-->
</style>";

/*** SET USER SELECTED STYLING ***/
# the following overrides default styles at runtime
# and prevents submitting the page with Return key
$opstyles = "
<!-- USER SELECTED FGW STYLING OVERRIDES -->
<style type='text/css'>
<!--
.fgw_container {";
if( !empty( $options['form_background'] ) )
	$opstyles .= "
	background-color:{$options['form_background']};";

if( !empty( $options['form_bgimg'] ) )
	$opstyles .= "
	background-image:url( {$options['form_bgimg']} );
	background-repeat:no-repeat;
	background-size:cover;";

if( !empty( $options['form_border'] ) )
	$opstyles .= "
	border:{$options['form_border']};";

if( !empty( $options['form_text_color'] ) )
	$opstyles .= "
	color:{$options['form_text_color']};";
$opstyles .= "
}";
if( !empty( $options['form_text_color'] ) )
	$opstyles .= "
	.fgw_container a:link, a:visited, a:hover {
		color:{$options['form_text_color']};
}";

if( !empty( $options['form_heading_color'] ) )
	$opstyles .= "
	.fgwheader {
		color:{$options['form_heading_color']};
	}";

if( !empty( $options['form_subhead_color'] ) )
	$opstyles .= "
	.fgw_container .subheading {
		color:{$options['form_subhead_color']};
}";

$opstyles .= "
.fgw_container input[type='text'],
.fgw_container select,
.fgw_container textarea,
.fgw_container .formcell {";
if( !empty( $options['field_text_color'] ) )
	$opstyles .= "
		color:{$options['field_text_color']};";

if( !empty( $options['field_background_color'] ) )
$opstyles .= "
	background-color:{$options['field_background_color']};";

if( !empty( $options['form_text_bold']) )
	$opstyles .= "
	font-weight:bold;";

$opstyles .= "
}";

if( !empty( $options['cell-border'] ) )
	$opstyles .= "
		.fgw_container .formcell {
   		border:{$options['cell_border']};
		}";

$opstyles .= "
.fgw_container .formcell .label {";

if( !empty( $options['cellhead_color'] ) )
   $opstyles .= "
   color:{$options['cellhead_color']};";

if( !empty( $options['cellhead_bkgd'] ) )
	$opstyles .= "
	background-color:{$options['cellhead_bkgd']};";

if( !empty( $options['cell_border'] ) )
   $opstyles .= "
   border-bottom:{$options['cell_border']};";
$opstyles .= "
}";

if( !empty( $options['form_labels'] ) )
	$opstyles .= "
.fgw_container .formlabel {
	color:{$options['form_labels']};
}";

if( !empty( $options['error_msg_color'] ) )
	$opstyles .= "
.fgw_container .warning {color:{$options['error_msg_color']};}";

if( !empty( $options['notify_msg_color'] ) )
	$opstyles .= "
.fgw_container .success {color:{$options['notify_msg_color']};}";

if( !empty( $options['husb_bkgd' ] ) )
	$opstyles .= "
#husbdata { background-color:{$options['husb_bkgd']};}";

if( !empty( $options['husb_img' ] ) )
	$opstyles .= "
#husbdata { background-image:url({$options['husb_img']});}";

if( !empty( $options['form_border'] ) )
	$opstyles .= "
#husbdata { border-top:{$options['form_border']};}";

if( !empty( $options['wife_bkgd' ] ) )
	$opstyles .= "
#wifedata { background-color:{$options['wife_bkgd']};}";

if( !empty( $options['wife_img' ] ) )
	$opstyles .= "
#wifedata { background-image:url({$options['wife_img']});}";

if( !empty( $options['form_border'] ) )
	$opstyles .= "
#wifedata { border-top:{$options['form_border']};}";

/* Javascript additions to the HTML header */
$opstyles .= "
-->
</style>
<script type='text/javascript'>
	var tnglitbox;
	var share = 0;
	var closeimg = '".$cms['tngpath']."$img/tng_close.gif';
	var smallimage_url = '".$cms['tngpath']."ajx_smallimage.php?';
	var cmstngpath='".$cms['tngpath']."';
	var loadingmsg = 'Loading...';
	var expand_msg = 'Expand';
	var collapse_msg = 'Collapse';
</script>
<base target='_parent'>";

$preferEuro = $tngconfig['preferEuro'] ? $tngconfig['preferEuro'] : "false";

// USER STYLING + ADDITIONAL SCRIPTING
$flags['scripting'] .= "
$opstyles
<script type='text/javascript'>
	<!--
	var newform = '{$text['ws_newform']}';
	var preferEuro = $preferEuro;
	var preferDateFormat = '';

	function stopRKey(evt) {
	  var evt = (evt) ? evt : ((event) ? event : null);
	  var node = (evt.target) ? evt.target : ((evt.srcElement) ? evt.srcElement : null);
	  if ((evt.keyCode == 13) && (node.type=='text'))  {return false;}
	}
	document.onkeypress = stopRKey;
	//-->
</script>
<script type='text/javascript' src='js/tngmenuhover2.js'></script>";

// Publish the header
tng_header( $title, $flags );

/***************************************************************
	DISPLAY NAVIGATION LINKS
***************************************************************/
// HANDLE POTENTIAL SESSION VAR NOTICES
$logged_in = !empty( $_SESSION['logged_in'] ) ? $_SESSION['logged_in'] : '';
$tngrole = !empty( $_SESSION['tngrole'] ) ? $_SESSION['tngrole'] : '';

// AUTHORIZED SITE VISITORS SEE MENU FOR CURRENT AND NEW WORKSHEETS
if( ( !$requirelogin && !$options['restricted'] ) || $logged_in ) 
{

	echo "
	 	<div id='navibar'>
			<a href='fgw.php' target='fgw'>{$text['ws_active']}</a>
			&nbsp;&nbsp;";

	echo "
			<a href='fgw.php?new=1' target='fgw'>{$text['ws_wslink']}</a>
			&nbsp;&nbsp;";
}

// SUPERUSERS SAME AS ABOVE + SEARCH AND LOGVIEW
if( !empty( $_SESSION['fgw_superuser'] ) ) 
{
	echo "
		<a href='fgw_search.php?'>{$text['ws_wslist']}</a>
		&nbsp;&nbsp;

		<a href='fgw_logview.php?'>{$text['ws_userlog']}</a>
		&nbsp;&nbsp;";
}

// ADMINS ONLY, SAME AS SUPERUSERS + SETUP
if( $tngrole == 'admin' ) 
{
	echo "
		<a href='fgw_setup.php?'>{$text['ws_wssetup']}</a>
		&nbsp;&nbsp;";

	// Current site/fgw access status
	$fgwpriv = !empty( $options['restricted'] ) ?
		'FGW '.$text['ws_closed'] : 'FGW '.$text['ws_open'];
	$tngpriv = !empty( $requirelogin ) ?
		'Site '.$text['ws_closed'] : 'Site '.$text['ws_open'];
	echo "
	<br />(<span style='white-space:nowrap;'><i>Status: $tngpriv, $fgwpriv</i></span>)
<br /><br />";
}

echo "
	</div>";

/***************************************************************
	START PUBLISHING THE FORM -- SHOW THE TITLE
***************************************************************/
echo "
<!-- Form Title -->
<div class='fgw_container'>
   <div class='asrow'>
      <div class='ascell icondiv'>
         <img src='$img/families.gif' width='24' height='24' alt='' />
      </div>
      <div class='ascell'>
         <span class='header fgwheader'>{$text['ws_fgw']}</span>
      </div>
   </div>";
/***************************************************************
	SHOW THE SUPERUSER'S ADMIN PANEL
***************************************************************/
if( $is_superuser ) 
{
   $merged_checked = empty( $form['merged'] ) ? null : "checked='checked'";
   $merged_style = $merged_checked ? null : 'background-color:yellow';
   $closed_checked = empty( $form['closed'] ) ? null : "checked='checked'";
   $closed_style = $closed_checked ? null : 'background-color:yellow';

	if( !isset($form ) ) 
	{
		$form = fgw_fetch_empty_record( $gedcom, $options['table'] );
	 	$form['updated'] = $text['ws_not_saved'];
		$disabled_admin_ctrls='disabled';
	}
	else 
	{
		$disabled_admin_ctrls ='';
	}

   echo "\n
<!-- Admin Section -->
<div class='admin compact' style='border:1px dashed #555;padding:5px;'>
   <h2 class='pagerow' style='text-align:center;margin:0;'>{$text['ws_management']}";
	// only show plain text link if the current worksheet has been saved
	if( !empty( $id ) && !empty( $fkey ) )
	{
		echo "
<br />
<a style='font-size:65%;' href='fgw_showplain.php?id=$id&amp;fkey=$fkey' target='_blank'>{$text['ws_linkplain']}</a>";
	}

	echo "
</h2>

<!-- Admin Section Line 1 Current worksheet status -->
<div class='pagerow subheading'>
	{$text['ws_currentws']} ({$text['ws_update']} [{$form['updated']}])
</div>

	<form action='fgw.php' id='f_admin' >
		<div class='formrow'>

            <!-- Worksheet (ID) -->
			<div class='formcell'>
				<div class='label'>{$text['ws_worksheet']}</div>
				<div style='height:24px;line-height:24px;width:100%;text-align:center;'>
					$id
				</div>
			</div>

            <!-- Merged (worksheet status flag set by admin) -->
			<div class='formcell' style='$merged_style'>
				<div class='label'>{$text['ws_merged']}</div>
				<input type='checkbox' class='living' name='merged' value='1' $merged_checked />
			</div>

            <!-- Closed (worksheet status flag set by admin -->
			<div class='formcell' style='$closed_style'>
				<div class='label'>{$text['ws_closed']}</div>
				<input type='checkbox' class='living' name='closed' value='1' $closed_checked />
			</div>

            <!-- FamilyID (set by FGW or by admin) -->
			<div class='formcell'>
				<div class='label'>{$text['ws_familyid']}</div>
				<input type='text' name='familyID' value='{$form['familyID']}' size='8' />
			</div>

            <!-- HusbandID (set by FGW or by admin) -->
			<div class='formcell'>
				<div class='label'>{$text['ws_husbandid']}</div>
				<input type='text' name='husbID' value='{$form['husbID']}' size='8' />
			</div>


            <!-- WifeID (set by FGW or by admin) -->
			<div class='formcell'>
				<div class='label'>{$text['ws_wifeid']}</div>
				<input type='text' name='wifeID' value='{$form['wifeID']}' size='8' />
			</div>
		</div>
";

// ALLOW ADMIN TO ADD/EDIT WORKSHEET TITLE - added in v43
	echo "
		<div class='formrow'>
			<div class='formcell'>
				<div class='label'>{$text['ws_wstitle']}</div>
				<input type='text' name='description' value=\"$formTitle\" size='59' />
			</div><!-- formcell -->
		</div><!-- formrow -->
";

	echo "
        <!-- Current worksheet status controls -->
		<div class='asrow'>
			<div class='ascell'>
				<input type='hidden' name='formID' value='$id' />
				<input type='submit' name='update' value='{$text['ws_update']}' $disabled_admin_ctrls />
			</div>

			<div class='ascell'>
				<button type='button' onclick=\"if (confirm('{$text['ws_confdel']}')) {window.location.href='fgw.php?formID=$id&amp;del=1';}\" $disabled_admin_ctrls>{$text['ws_delete']}</button>
			</div>
		</div>
	</form><!-- Admin Line 1 -->
   <div class='clear'></div>";

// Merge Button
	if( !empty($data['mergit']) && ( empty( $form['merged'] ) && empty($form['closed'] ) ) ) 
	{
		$disabled_merge = '';
		$nomerge = '';
	}
	else 
	{
		$disabled_merge = 'disabled';
		$nomerge = '&nbsp;&nbsp;Auto-merge not available for this form';
	}

   	echo "\n
   <!-- Admin Section Line 2 -->
	<div class='pagerow subheading'>
         {$text['ws_otherws']}
	</div>
	<div>
   	<!-- Admin Line 2 Get different worksheet -->
		<form action='fgw_search.php' method='post' id='f_search' targt='fgw' >
		<input type='submit' name='search'
			value='{$text['ws_searchbyid']}'
		title='familyID, husbandID, wifeID or worksheetID' $disabled_admin_ctrls/>

		<input type='text' id='searchID' name='searchID' size='10' />
		<input type='hidden' name='backformID' value='$id' />
		<input type='hidden' name='backfkey' value='$fkey' />
		<input type='hidden' name='backtitle' value='{$form['description']}' />
	</form>
	</div>";

	$all = $pending = $merged = $closed = '';
	if( !empty( $_SESSION['filter'] ) ) 
	{
		${$_SESSION['filter']} = "checked='checked'";
	}
	else 
	{
		$all = "checked='checked'";
	}

	$murl = $tngdomain.'/'.$fgw_path.'fgw.php?id='.$id.'&amp;fkey='.$fkey;
	echo "\n
	<!-- Admin Section Line 3 Navigation filter -->
	<form action='fgw.php' method='post' id='f_nav' >
	<div class='astable'>
		<div class='asrow'>
			<div class='ascell subheading'>{$text['ws_filter']}</div>
		</div>
	</div>

	<!-- navigation filter selections table -->
	<div class='astable'> 
		<div class='asrow'>
		
			<!-- Get next/previous worksheet -->
			<div class='ascell'>
				<label for='f1'>&nbsp;<span>{$text['ws_allforms']} </span>
				<input type='radio' id='f1' class='' name='filter'
					value='all' $all/>
				</label>
		 		&nbsp;&nbsp;&nbsp;&nbsp;
			</div>

			<!-- Get next/previous Open worksheet -->
			<div class='ascell'>
				<label for='f2'><span>{$text['ws_pending']} </span>
				<input type='radio' id='f2' class='' name='filter'
					value='pending' $pending/>
				</label>
				&nbsp;&nbsp;&nbsp;&nbsp;
			</div>

			<!-- Get next/previous Merged worksheet -->
			<div class='ascell'>
				<label for='f3'><span>{$text['ws_merged']} </span>
				<input type='radio' id='f3' class='' name='filter'
					value='merged' $merged/>
				</label>
				&nbsp;&nbsp;&nbsp;&nbsp;
			</div>

			<!-- Get next/previous Closed worksheet -->
			<div class='ascell'>
			<label for='f4'><span>{$text['ws_closed']} </span>
				<input type='radio' id='f4' class='' name='filter'
					value='closed' $closed/>
			</label>
			</div>
			
		</div>
		
		<div class='asrow'><div class='ascell'>&nbsp;</div></div>
		
	</div><!-- astable -->

    <!-- Navigation filter controls table -->
	<div class='astable'>
		<div class='asrow'>
			<div class='ascell'>
				<input type='submit' name='prev' 
					value='&laquo; {$text['ws_prev']}' $disabled_admin_ctrls/>
				<input type='submit' name='next' 
					value='{$text['ws_next']} &raquo;' $disabled_admin_ctrls/>
				<input type='hidden' name='formID' value='$id' />
			</div>
		</div>

		<!-- Copyable link to this worksheet -->
		<div class='asrow'>
			<br />
			<div class='ascell' style='float:right'>
				<button type='button' onclick=\"copyToClipboard('link1')\" 
					$disabled_admin_ctrls>{$text['ws_copylink']}</button>
			 	&nbsp;&nbsp;<span id='link1' style='text-decoration:underline;'>$murl</span>
			</div>
		</div>
	</div>
	</form>";

   echo "\n
<div class='clear'></div>
</div><!-- Admin Section -->

<div class='clear'></div>
";
} // superuser's worksheet management panel

/***************************************************************
	DISPLAY THE FAMILY GROUP WORKSHEET
***************************************************************/
echo "
<!-- FAMILY HISTORY -->
<form action='fgw_worksheet.php' method='post' name='worksheet' id='worksheet' >
<div id='fgwfgw'>
	<div id='fgwfamily'>";

$confirmation = 'e35325117ab18226b4ddf6c780a30acf';

// INSERT MAIN FAMILY GROUP WORKSHEET FORM
require 'fgw_form.php';
	echo "
	</div><!--fgwfamily-->";
/***************************************************************
NOTES TABLE
***************************************************************/
if( !isset( $unotes ) ) $unotes = '';
echo "
<div class='pagerow subheading'>
     {$text['ws_title_notes']}
</div>
<div class='pagerow'>
   <div>
      <textarea name='unotes' rows='4'>$unotes</textarea>
   </div>
</div>
";
/***************************************************************
SUBMITTER'S INFORMATION TABLE
***************************************************************/
if( !isset( $urelation ) ) $urelation = '';

echo "
<div id='usertab'><!--submitter information-->
<div class='pagerow subheading'>
     {$text['ws_submittedby']}*
</div>

<div class='formrow'>
   <div class='formcell'>
      <div class='label'>{$text['ws_yourname']} <span class='{$options['error_msg_color']}'>({$text['ws_required']})</span></div>
      <input class='fullname' name='user[realname]' value='{$user['realname']}' type='text' />
   </div>

   <div class='formcell'>
      <div class='label'>{$text['ws_city']}</div>
      <input class='city' name='user[city]' value='{$user['city']}' type='text' />
   </div>

   <div class='formcell'>
      <div class='label'>{$text['ws_state']}</div>
      <input class='state' name='user[state]' value='{$user['state']}' type='text' />
   </div>

   <div class='formcell'>
      <div class='label'>{$text['ws_country']}</div>
      <input class='country' name='user[country]' value='{$user['country']}' type='text' />
   </div>

   <div class='formcell'>
      <div class='label'>{$text['ws_email']}  <span class='warning'>({$text['ws_required']})</span></div>
      <input class='email' name='user[email]' value='{$user['email']}' type='text' />
   </div>

   </div>
</div><!--#usertab-->
<div class='asrow'>
	<div class='ascell'>
		<small>*{$text['ws_consent']}</small>
	</div>
</div>

<div class='pagerow subheading'>
     {$text['ws_relationship']}
</div>
<div class='pagerow'>
      <textarea name='urelation' rows='4'>$urelation</textarea>
</div>

<div class='pagerow subheading'>
      {$text['ws_title_credits']}
</div>
<div class='pagerow'>
      <textarea name='user[credit]' rows='1'>{$user['credit']}</textarea>
</div>
</div><!-- #fgwfgw -->";
/* User changes to worksheet are not tracked after this point */

// added in version 43
$tngrole = isset( $_SESSION['tngrole'] ) ? $_SESSION['tngrole'] : '';
$admin = $tngrole == 'admin' ? "ADMIN: " : "";

/* do not require superusers to deal with captcha */
if( !$is_superuser && !empty( $options['show_captcha'] ) ) 
{
   echo "
<div class='pagerow'>&nbsp;</div>
<div class='asrow'>
	<div class='ascell' style='vertical-align:middle;'>
		{$text['ws_security']}
		<span class='{$options['error_msg_color']}'>({$text['ws_required']})</span>:

	</div>
	<div class='ascell' style='vertical-align:middle;'>
		<input type='text' id='capcode' name='capcode' size='5' value='' />
		<input type='hidden' id='fgw_hash' name='fgw_hash' value='$fgw_hash' />
	</div>
	<div class='ascell'>
		<img src='fgw_captcha.php' />
	</div>
</div>
";
}
//echo __LINE__,' ',$mod_support;exit;
/***************************************************************
FORM SUBMISSION CONTROLS TABLE
***************************************************************/
//$mm_checked = empty( $sent )  || $mailme ? "checked='checked'" : "";
//$href = 'fgw.php';		//$_SERVER['PHP_SELF'];
//$href .= $_SESSION['fgw_guest'] ? '?vs=1' : "?tree=$gedcom";

echo "
<div class='pagerow compact'>
   <div style='text-align:center;'>
      &nbsp;
   </div>
</div>

<div class='pagerow compact'>
   <div class='pagecell'>
      <input type='submit' value='{$text['ws_savesubmit']}' id='submitted' name='submitted' />
      <input name='formID' value='$id' type='hidden' />
      <input type='hidden' name='persfamID' value='$persfamID' />
   </div>
</div>

<div class='pagerow compact'>
   <div style='text-align:center;'>
      <small><i>{$text['ws_form']} ver $fgw_version, rev $revdate</i></small>
   </div>
</div>

<div id='comments' >
  Do not enter anything into this field
	<input type='text' name='comment' value='' />
</div>
</form><!--worksheet-->

</div><!--fgw_container-->
<div style='display:none;position:sticky;bottom:15%;' id='wait'>
  <img src='$img/ringspin.gif' alt='' />
</div>
";
/***************************************************************
	MESSAGE CACHES FOR JAVASCRIPT DISPLAYS
***************************************************************/
/* Cached messages for javascript Alerts */
echo "
<div id='newtree' style='display:none'>{$text['ws_newtree']}</div>
<div id='nochanges' style='display:none'>{$text['ws_nochanges']}</div>
<div id='childline' style='display:none'>{$text['ws_childline']}</div>
<div id='nocode' style='display:none'>{$text['ws_nocode']}</div>";

/* Cashed for jQuery processing */
echo "
<div id='superuser' style='display:none'>$is_superuser</div>";
/***************************************************************
	FOOTER SCRIPTS
***************************************************************/
echo "
<script type='text/javascript' src='js/datevalidation.js'></script>
<script type='text/javascript' src='js/litbox.js'></script>
<script type='text/javascript' src='js/fgw_lib.js?v=22'></script>";

/*
 * Following javascript is used in the worksheet's Admin section to allow
 * the admin to copy the existing id+fkey worksheet link to the clipboard,
 * then past it in an email or elsewhere.
 *
 * We had to modify ~line 1180 to change last element .innerHTML to
 * .textContent to prevent html changing the '&' to '&amp;' which changed
 * the fieldname 'fkey' to '&amp;fkey' and broke the url.
 */
echo "
<script type='text/javascript'>
// pure javascript
function copyToClipboard(elementId) {
  		// Create a 'hidden' input
  var aux = document.createElement('input');
  		// Assign it the value of the specified element
	aux.setAttribute('value', document.getElementById(elementId).textContent);
  		// Append it to the body
  document.body.appendChild(aux);
  		// Highlight its content
  aux.select();
  		// Copy the highlighted text
  document.execCommand('copy');
  		// Remove it from the body
  document.body.removeChild(aux);
}
</script>";

tng_footer(''); 
exit;
