<?php
/***************************************************************
	fgw_ajax_proc.php
	FAMILY GROUP WORKSHEET AJAX EMAILER

	Permits admin to save empty or partially filled worksheet.
	Does not prefill worksheet with admin's personal data.
	If submitter's name and email are missing, no email is sent.

	Upon form submissin, jQuery function sends form data to fgw_ajax_proc.php for 	processing and databasing.

	if processing is successful fgw_ajax_proc.php sends a three-part message back
	to the jQuery function containing 1) a text message, 2) the worksheet ID and 3)
	the database access key for the processed form. After the message is read by
	the user, jQuery refreshes the screen to pick up the new id and fkey.

	If there is a processing problem (like the user did not provide his email
	address), a one-part message is sent back to jQuery for display, then jQuery
	stays on the worksheet to allow the user to fix the problem.

	NOTE ON MESSAGE NUMBERS: When processing a worksheet, proc status
	is returned back to the fgw_lib.js submit function and displayed
	along with the message.  The message starts with a hyphenated number
	in the form of 88-288; to assist in trouble shooting, the first segment
	is the line number processed in fgw_lib.js, and the second is the line
	in this script that issued the message.
***************************************************************/
require_once 'tng_begin.php';
require_once 'fgw_inits.php';

// DETECT ATTEMPTED DIRECT ACCESS TO THIS SCRIPT
if( empty( $_POST ) || !is_array( $_POST ) ) {
	require_once $admin_language_file;
	trigger_error( $admtext['wse_access'], E_USER_ERROR );
	die();
}

# Essential for proper rendering of special chars in supported languages
$thecharset = $session_charset;
header("Content-Type:text/html; charset=$thecharset");
ini_set( 'default_charset', $thecharset );

require_once 'fgw_dbconn.php';
require_once 'fgw_lib.php';
require_once 'fgw_logger.php';
require_once $cms['tngpath'].'tngmaillib.php';

$options = fgw_sanitize_recursive( $options );
foreach( $options as $key => $value ) {
    ${$key} = $value;
}
//$fgw_logstring = $text['ws_submitted'];

/***************************************************************
SAVE POSTED DATA TO DATABASE WORKSHEET
***************************************************************/

if( !isset( $_POST ) || !is_array($_POST) ) exit;

foreach ($_POST as $key => $value) {
	// separate temp security data from form to be saved
	if( $key == 'capcode' || $key == 'fgw_hash' ) {
		${$key} = $value;
		continue;
	}
  	$post[$key] = ${$key} = fgw_sanitize_recursive( $value );
}

if(!empty($comment)) { // spambot fell for the honeypot}
  fgw_writelog($options['userlog'], "Spambot:wse_noaccess:",
  			$options['max_loglines'], $options['nologip'] );
	trigger_error( 'Spambot!', E_USER_ERROR );
	die();
}

/*
* Allow superuser to submit empty form without submitter info and captcha.
 * Others must provide info and captcha if required in FGW setup.
*/
if( empty( $_SESSION['fgw_superuser'] ) ) {
	if( empty( $husb['firstname'] ) || empty( $husb['lastname'] ) ) {
	  if( empty( $wife['firstname'] ) || empty( $wife['lastname'] ) ) {
	    echo $text['ws_noinfo'];
	    exit;
	  }
	}

	if( empty( $user['realname'] ) || empty( $user['email'] ) ) {
		echo $text['ws_need_email'];
	  exit;
	}

	if( !empty( $options['show_captcha'] ) ) {
   	$newhash = md5( strtolower($capcode));
	   if( $newhash != $fgw_hash ) {
	   	echo $text['ws_nocode'];
	      exit;
	   }
	}
}

$_SESSION['fgw_user'] = $user;

$formTitle = fgw_title( $husb, $wife );
# all okay to this point -- save the form data

$familyID = empty( $_POST['marr']['familyID'] ) ? '' : $_POST['marr']['familyID'];

// remove empty child lines
$kids = array();
foreach( $post['child'] as $kid ) {
   if( !empty( $kid['firstname'] ) ) {
      $kids[] = $kid;
   }
}
$post['child'] = $kids;


//add famc to new kids
// foreach( $post['child'] if no personID famc = marr['familyID']
for( $i=0; isset($post['child'][$i]); $i++ ) {
   if( empty( $post['child'][$i]['famc'] ) ) {
      $post['child'][$i]['famc'] = $post['marr']['familyID'];
   }
}

$serialized = base64_encode( serialize( $post ) );
$serialized = db_escape( $serialized );

$formID = $post['formID'];

// UPDATE EXISTING DATABASE WORKSHEET
if( !empty( $formID ) ) {
   $query = "UPDATE {$options['table']}
	set data='$serialized', email='{$user['email']}',
	subname='{$user['realname']}', description='$formTitle',
	familyID='{$marr['familyID']}', husbID='{$husb['personID']}',
	wifeID='{$wife['personID']}', merged=0
	WHERE formID=$formID";
	db_update( $query );
	$logstring = "fgw#$formID:ws_updatedby:";
	fgw_writelog( $options['userlog'], $logstring,
				$options['max_loglines'], $options['nologip'] );
}

// SAVE NEW WORKSHEET TO DATABASE
else {
  // set date & time the worksheet is created
  $created = date( "Y-m-d H:m:s");

  // create a unique form key from the date/time
  $fkey = md5( $created );

  // ADD THE WORKSHEET
  $query = "INSERT INTO {$options['table']}
   (gedcom,description,data,email,subname,created,fkey,familyID, husbID, wifeID)
      VALUES
   ( '$gedcom', '$formTitle', '$serialized', '{$user['email']}','{$user['realname']}',
   '$created','$fkey','$familyID', '{$husb['personID']}', '{$wife['personID']}')";

  $formID = db_insert( $query );
  $logstring = "fgw#$formID:ws_savedby:";
  fgw_writelog( $options['userlog'], $logstring,
  				$options['max_loglines'], $options['nologip'] );
}
/*
$_SESSION['fgw_page8'] = $tngdomain.'/'.$fgw_path."fgw.php?id=$formID";
*/
$_SESSION['fgw_id'] = $formID;
//echo __LINE__;exit;

// RESET THE MERGED FLAG
$query = "UPDATE {$options['table']} set merged=0
  WHERE formID=$formID";
db_update( $query );

// SET USER NAME IF LOGGED IN
$user['username'] = empty( $_SESSION['currentuser'] )
  ? 'not logged in'
  : $_SESSION['currentuser'];

// RETRIEVE WORKSHEET AND FKEY FROM DATABASE TO VERIFY THEM AND INCLUDE FKEY IN EMAILS
$form = fgw_fetch_form_record( $formID );
$fkey = $form['fkey'];
$_SESSION['fgw_id'] = $formID;
$_SESSION['fgw_fkey'] = $fkey;

// if admin's realname OR email missing - don't send email
if( !empty( $_SESSION['fgw_superuser'] ) ) {
	if(empty($user['realname']) || empty($user['email'])) {
		echo $formID,'|',$fkey,'|',__LINE__,": FormID=$formID\nAdmin: {$text['ws_saved']}";
	   exit;
	}
}

//	EACH FORM-SAVE GENERATES MESSAGE TO ADMIN/SUBMITTER ONCE PER SESSION
if( !empty( $_SESSION['fgw_mailed'] ) ) {
   $fgw_mailed = $_SESSION['fgw_mailed'];
   foreach( $fgw_mailed as $mailed ) {
      if( $mailed == $formID ) {
			echo $formID,'|',$fkey,'|',__LINE__,': ',$text['ws_saved'];
        exit;
      }
   }
}

// SAVE THE NEW WORKSHEET ID TO A SESSION ARRAY AND CONTINUE WITH EMAIL
$_SESSION['fgw_mailed'][] = $formID;

/*************************************************************************
CONSTRUCT EMAIL MESSAGE BODY
*************************************************************************/
# credit string $user['credit'] formed in fgw.php?
$credit_string = empty( $user['realname'] ) ? "" : $user['realname'];

//$credit_string.= empty($user['city'] ) ? "" : empty( $credit_string ) ? $user['city'] : ", {$user['city']}";
if( empty($user['city'] ) ) $credit_string .= '';
elseif( empty( $credit_string ) ) $credit_string .= $user['city'];
else $credit_string .= ", {$user['city']}";

//$credit_string.= empty($user['state'] ) ? "" : empty( $credit_string ) ? $user['state'] : ", {$user['state']}";
if( empty($user['state'] ) ) $credit_string .= '';
elseif( empty( $credit_string ) ) $credit_string .= $user['state'];
else $credit_string .= ", {$user['state']}";

//$credit_string.= empty($user['country'] ) ? "" : empty( $credit_string ) ? $user['country'] : ", {$user['country']}";
if( empty($user['country'] ) ) $credit_string .= '';
elseif( empty( $credit_string ) ) $credit_string .= $user['country'];
else $credit_string .= ", {$user['country']}";

# if no trailing slash in website URL as configured, append one
$tngdomain = ( mb_substr( $tngdomain, -1, 1, $session_charset ) != '/' ) ?
$tngdomain .'/' :
$tngdomain;

# Start building message -- Title Section
$message = $formTitle . "\n";
$message.= "{$text['ws_submitted_to']}: $sitename ($tngdomain)\n";
$message.= empty( $credit_string ) ? "" : "{$text['ws_author']}: $credit_string ({$user['email']})\n";
$message.= empty( $urelation ) ? "" : $urelation."\n";
# Login Section
$message.= "\n";
$message.= strtoupper( $text['ws_login'] ) . "\n";
$message.="{$text['ws_user']}: ";
$message.=$user['username'] ? $user['username']."\n" : "not logged in\n";
$message.=$user['realname'] ? $text['ws_name'].': '.$user['realname']."\n" : "";
$message.=$user['email'] ? $text['ws_email'].': '.$user['email']."\n" : "";

$message .= "\n{$text['ws_familytree']}: ";
$message .= fgw_fetch_treename( $gedcom );
$message .= "\n\n";
$message .= $text['ws_link'].":\n";

// INSERT LINK BACK TO STORED WORKSHEET
$message .= $tngdomain.'/fgw.php?id='.$formID.'&amp;fkey='.$fkey;

$message.="\n\n";

/*************************************************************************
SEND THE EMAIL MESSAGE SEPARATELY TO THE ADMIN AND THEN TO THE SUBMITTER
*************************************************************************/
$message .= $text['ws_thanks'];

// SET ADMIN ADDRESS TO USE -- ALTERNATE DEFINED IN SETUP.PHP AND STORED IN USER_CONFIG.PHP
$adminmailtouse = $alt_addr ? $alt_addr : $emailaddr;
$adminnametouse = $alt_name ? $alt_name : ($sitename ? $sitename : $dbowner);


if( !empty( $familyID ) ) 
	$persfam = '('.$familyID.')';
elseif( !empty( $husb['personID'] ) )
	$persfam =  '('.$husb['personID'].')';
elseif( !empty( $wife['personID'] ) )
	$persfam =  '('.$wife['personID'].')';
else
	$persfam = null;

# send message to site administrator
$sent = tng_sendmail($user['realname'], $user['email'], $adminnametouse, $adminmailtouse, $text['ws_fgw']." $persfam", $message, $user['email'], $user['email']);

# if all is well, send a copy back to the contributor
if ( $sent )
    $sent = tng_sendmail($adminnametouse, $adminmailtouse, $user['realname'], $user['email'], $text['ws_fgw']." $persfam", $message, $adminmailtouse, $adminmailtouse);

# compose on-screen status message
if ($sent) {
    if( !empty( $_SESSION['fgw_newform'] ) ) {
        //refresh screen after after first submission
        unset( $_SESSION['fgw_newform'] );
        echo $formID,'|',$fkey,'|',__LINE__,':',$text['ws_thanks'];
    }
    else {
		 echo $formID,'|',$fkey,'|',__LINE__,': ',$text['ws_thanks'];
    }
}
else {
	echo $formID,'|',$fkey,'|',__LINE__,': ',$text['ws_notsent'];
}
?>
