<?php

/*
 * Include file after mytng_dbcomm which takes care of db errors
 * Functions provide i/o and data manipulation for TNG database
 *
 * Script should have its own language support independent of app
 * GENERALIZE IT FOR ALL TNG SCRIPTS INDEPENDENT OF APP
 *
*/
// DETECT ATTEMPTED DIRECT ACCESS TO THIS INCLUDE-ONLY SCRIPT
if( count(get_included_files()) == 1 ) {
	require_once 'fgw_inits.php';
	trigger_error( $admtext['wse_access'], E_USER_ERROR );
	die();
}

if( !isset( $DB  ) ) {
	trigger_error( $admtext['wse_no_dbconn'], E_USER_ERROR );
	die();
}

/*
// error messages
define( 'NODB_ERR',     'TNG root not found!' );
define( 'NOCONN_ERR',	'Database not connected!');
define( 'QTYPE_ERR',    'Unrecognized query type!' );
define( 'SELECT_ERR',   'Unable to select record!'  );
define( 'INSERT_ERR',   'Unable to insert record!' );
define( 'DELETE_ERR',   'Unable to delete record!' );
define( 'REPLACE_ERR',  'Failed to replace record!');
define( 'UPDATE_ERR',   'Failed to update record!' );
define( 'NOREC_ERR',    'Record not found!' );
define( 'ARGS_ERR',     'Missing arguments!' );
define( 'COLNAMES_ERR', 'Failed to retrieve table from MySQL!');
define( 'IDTYPE_ERR',   'Unrecognized ID Type!' );
define( 'NEXTID_ERR',   'Failed to get next ID number!');
*/
/*************************************************************************
COMMON TNG DATA & DATABASE FUNCTIONS
*************************************************************************/
function fetch_column_names( $table ) {
	$names = array();
  	$rows = db_select( "DESCRIBE $table" );
	if( !count( $rows ) ) {
		$backtrace = debug_backtrace();
		trigger_error( COLNAMES_ERR, E_USER_ERROR );
	}
	foreach( $rows as $row ) {
		$names[] = $row['Field'];
	}
	return $names;
}

function fetch_table_data( $table ) {
	$tabdata = array();
	$rows = db_select( "DESCRIBE $table" );
	if( !count( $rows ) ) {
	    $backtrace = debug_backtrace();
	    edlog( $backtrace, COLNAMES_ERR );
	    trigger_error( COLNAMES_ERR, E_USER_WARNING );
  	}
	return $rows;
}

function fetch_next_id( $type, $gedcom ) {
  global $people_table, $families_table, $sources_table, $repositories_table, $options;
	switch( $type ) {
		case $options['pid_prefix']:
			$id_name = "personID";
			$table = $people_table;
			break;
		case $options['fid_prefix']:
			$id_name = "familyID";
			$table = $families_table;
			break;
		case "S":
			$id_name = "sourceID";
			$table = $sources_table;
			break;
		case "R":
			$id_name = "repoID";
			$table = $repositories_table;
			break;
		default:
        edlog( $backtrace, IDTYPE_ERR );
        trigger_error( IDTYPE_ERR, E_USER_WARNING );
			return "";
			break;
	}
  $newID = -1;
  $query = "SELECT MAX(0+SUBSTRING( $id_name, 2 ) ) as newID
    FROM $table
    WHERE gedcom='$gedcom' AND $id_name LIKE '$type%'";
  $rows = db_select( $query );
  $newID = $rows[0]['newID'];
  if( !$newID ) {
    $backtrace = debug_backtrace();
    trigger_error( $backtrace, E_USER_ERROR );
  }
  $newID++;
	return "$type$newID";
}

function get_empty_rec( $table ) {
  $rec = array();
  $names = fetch_column_names( $table );
  foreach( $names as $name ) {
    $rec[$name] = '';
  }
  return $rec;
}

// REQUIRES DBCONN FOR DATABASE FUNCTIONS
// @table - db table to insert data into
// @data - array of data as fieldname => value
function prepare_insert_query( $table, $data ) {
	global $currentuser;
	$trows = fetch_table_data( $table );
	$data_keys = array_keys( $data );
	$data = db_escape( $data );
	$record = array();

	foreach( $trows as $row ) {
		$key = $row['Field'];
		if( in_array( $key, $data_keys ) ) {
			$record[$key] = $data[$key];
		}
		elseif( $row['Null'] == 'YES' ) {
			$record[$key] = NULL;
		}
		else {
			$type = $row['Type'];
			$parts = explode( "(", $type );
			$subtype = $parts[0];
			switch ($subtype) {
				case 'int':
				case 'tinyint':
				case 'smallint':
				case 'mediumint':
				case 'bigint':
				case 'float':
				case 'double':
				case 'decimal':
					$record[$key] = 0;
					break;
				case 'char':
				case 'varchar':
				case 'text':
				case 'tinytext':
				case 'blob':
					$record[$key] = '';
					break;
				case 'date':
					if( preg_match( "/datetr$/", $key ) ) {
						$record[$key] = datestr_to_datetr(
							$record[substr( $key, 0, -2 )] );
					}
					else
						$record[$key] = '';
					break;
				case 'datetime':
					$record[$key] = '1000-00-00 00:00:00';
					break;
				case 'timestamp':
					$record[$key] = '1970-01-01 00:00:01.000000';
					break;
				default:
					break;
			}
		}
	}

	$field_names = array_keys( $record );

	// take care of changedby & changedate fields if any
	if( in_array( "changedby", $field_names ) )
		$record['changedby'] = $currentuser;
	if( in_array( "changedate", $field_names ) )
		$record['changedate'] = date( "Y-m-d H:i:s" );

	// construct the mysql field and data strings
  $fieldstr = $datastr = '';
	foreach( $record as $key => $value ) {
		$fieldstr .= "$key, ";
		$datastr .= "'$value', ";
	}
  // remove the unneeded comma-space from end of strings
	$fieldstr = substr($fieldstr, 0, -2);
	$datastr = substr($datastr, 0, -2);

	$query = "INSERT INTO $table ($fieldstr) VALUES ($datastr)";
  return $query;
}

function prepare_replace_query( $table, $row, $data) {
  global $currentuser;
  $data_fields = array_keys( $data );
  $columns = array_keys( $row );
  $data = db_escape( $data );

  // only add $data values to row if their keys = table columns - ignore the rest
  foreach( $row as $key => $value ) {
    if( in_array( $key, $data_fields ) ) {
      $row[$key] = $data[$key];
      if( preg_match( "/date$/", $key ) ) {
        if( in_array( $key.'tr', $columns ) ) {
          $row[$key.'tr'] = datestr_to_datetr( $row[$key] );
        }
      }
    }
    elseif( $key == "changedate" ) {
      $row[$key] = date( "Y-m-d H:i:s" );
    }
    elseif( $key == "changedby" ) {
      $row[$key] = $currentuser;
    }
  }
  // create the columns-values lists
  $fieldstr = $valuestr = '';
  foreach( $row as $key => $value ) {
    $fieldstr .= "$key,";
    $valuestr .= "'$value',";
  }
  $fieldstr = rtrim($fieldstr, ",");
  $valuestr = rtrim($valuestr, ",");

  $query = "REPLACE INTO $table ($fieldstr) VALUES ($valuestr)";
  return $query;
}

//***** FORMATTING *****//
function format_id( $prefix, $id ) {
	$id = trim( $id );
	$id = is_numeric( $id ) ? $prefix . $id : $id;
  if( !is_numeric( substr( $id, 1 ) ) ) return '';
  return ucfirst( $id );
}

function format_data( $array ) {
	foreach( $array as $key => $value ) {
		switch( $key ) {
			case 'personiID':
			case 'husband':
			case 'wife':
				$array[$key] = format_id( "I", $value );
				break;
			case 'familyID':
				$array[$key] = format_id( "F", $value );
				break;
			case 'sourceID':
				$array[$key] = format_id( "S", $value );
				break;
			case 'birthdate':
			case 'baptdate':
			case 'altbirthdate':
			case 'burialdate':
			case 'deathdate':
			case 'marrdate';
			case 'divdate':
				$array[$key] = ucfirst( $value );  // for Abt, Bef etc
				break;
			default:
				break;
		}
	}
	return $array;
}

function ret_date_place_string($date, $place) {
	if (empty ($date))
		$retstr = $place;
	else if (empty ($place))
		$retstr = $date;
	else
		$retstr = "$date, $place";
	return $retstr;
}

function datestr_to_datetr( $datestr ) {
  $months = array( 'Jan','Feb','Mar','Apr','May','Jun',
                  'Jul','Aug','Sep','Oct','Nov', 'Dec' );
  // EXTRACT YEAR
  $pat1 = "/(.*)([0-9]{4})$/";
  // EXTRACT MONTH AS TEXT IF EXISTS
  $pat2 = "/([a-zA-Z]{3})/";
  // EXTRACT DATE AS DIGITS IF EXISTS
  $pat3 = "/^([0-9]{1,2})/";

  $y = "0000"; $m = "00"; $d = "00";
  if( !empty( $datestr ) ) {
    // YEAR
    if( preg_match( $pat1, $datestr, $match ) ) {
      $y = $match[2];
      $tmp = $match[1];
    }
    else
      $tmp = '';
      // MONTH AS TEXT
    if( preg_match( $pat2, $tmp, $match ) ) {
      $mon = $match[1];
      if( in_array( $mon, $months ) ) {
        $m = array_search( $mon, $months ) + 1;
        $m = sprintf( "%02d", $m );
      }
    }
    // DAY
    if( preg_match( $pat3, $tmp, $match ) ) {
      $d = $match[1];
      $d = sprintf( "%02d", $d );
    }
  }
  return "$y-$m-$d";
}

function rand_string( $length ) {
  // usage: $string = rand_string( 10 );
	$chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";

	$size = strlen( $chars );
	for( $i = 0; $i < $length; $i++ ) {
		$str .= $chars[ rand( 0, $size - 1 ) ];
	}

	return $str;
}

function get_input_args( $input_array ) {
  global $session_charset, $options;
  $magic_quotes = get_magic_quotes_gpc() == 0 ? false : true;
	$args = array();
	foreach( $input_array as $key=>$value ) {
    $args[$key] = $magic_quotes ? trim($value) : trim( addslashes($value) );
		switch( $key ) {
			case 'husband':
        $args['husband'] = format_id( $options['pid_prefix'], $value );
        break;
			case 'wife':
				$args['wife'] = format_id( $options['pid_prefix'], $value );
        break;
			case '_pid':
			case 'personID':
				$args[$key] = format_id( $options['pid_prefix'], $value );
				break;
			case '_sid':
			case 'sourceID':
				$args[$key] = format_id( "S", $value );
				break;
			case 'familyID':
				$args[$key] = format_id( $options['fid_prefix'], $value );
				break;
			default:
				break;
		}
	}
	return $args;
}

//***** SESSIONS*****//
function pad_missing_keys( $vars ) {
  global $vars_table;
  foreach( $vars_table as $table_key => $sess_key ) {
    if( !array_key_exists( $table_key, $vars ) )
      $vars[$table_key] = '';
  }
  return $vars;
}

function set_session_var( $key, $value ) {
  global $vars_table;
  // if in the vars table use translation to set it
	if( array_key_exists( $key, $vars_table ) ) {
		$_SESSION[$vars_table[$key]] = $value;
    return $_SESSION[$vars_table[$key]];
	}
  else {
    // set the session variable without translation
    $_SESSION[$key] = $value;
    return $_SESSION[$key];
  }
}

function set_session_vars( $args ) {
  global $vars_table, $options;
  // only set session vars if they are in the translation table
	foreach ($vars_table as $args_key => $sess_key) {
		if (array_key_exists ($args_key, $args)) {
			if ($args_key == 'personID') $args['personID'] = format_id( $options['pid_prefix'], $args['personID']);
			if ($args_key == 'sourceID') $args['sourceID'] = format_id( 'S', $args['sourceID']);
			//	if key exists, but vaue is empty, unset the session var
			if (empty ($args[$args_key]))
				unset ($_SESSION[$sess_key]);
			else
				$_SESSION[$sess_key] = $args[$args_key];
		}
	}
}

function get_session_var( $var ) {
  global $vars_table;
  // if in the vars table use translation to fetch it
	if( array_key_exists( $var, $vars_table ) ) {
		if( isset( $_SESSION[$vars_table[$var]] ) )
			return $_SESSION[$vars_table[$var]];
  }
  else
    // fetch it without translation
    if( isset( $_SESSION[$var] ) )
      return $_SESSION[$var];
	return NULL;
}

function get_session_vars() {
  global $vars_table;
  // only fetch session vars if they are in the translation table
	$args = array();
	foreach( $vars_table as $args_key => $sess_key )
		if( isset( $_SESSION[$sess_key] ) )
			$args[$args_key] = $_SESSION[$sess_key];
    else {
      $args[$args_key] = '';
    }
	return $args;
}

function unset_session_vars() {
	global $vars_table;
	foreach( $vars_table as $args_key => $sess_key ) {
		unset( $_SESSION[$sess_key] );
  }
}

function unset_session_var( $var ) {
	global $vars_table;
	if( array_key_exists( $var, $vars_table ) )
		unset( $_SESSION[$vars_table[$var]] );
}

function merge_session_vars( $args ) {
  global $vars_table;
	foreach( $vars_table as $args_key => $sess_key )
		if( isset( $_SESSION[$sess_key] ) )
			$args[$args_key] = $_SESSION[$sess_key];
	return $args;
}

function sanitize_recursive( $variable ) {

   if ( is_string( $variable ) ) {
      $variable = trim($variable);
      $variable = htmlspecialchars($variable);
      return stripslashes( $variable );
   }
   if ( is_array( $variable ) ) {
      foreach( $variable as $i => $value ) {
         $variable[ $i ] = sanitize_recursive( $value );
      }
   }
   return $variable ;
}

/*************************************************************************
ADDRESSES
*************************************************************************/
function fetch_address_rec( $addressID ) {
  global $address_table;
  if( $addressID ) {
    $query = "SELECT * FROM $address_table WHERE addressID='$addressID'";
    return db_select( $query );
  }
}

function insert_address_rec( $data ) {
  global $address_table;
  $query = prepare_insert_query( $address_table, $data );
  return db_insert( $query );
}

function update_address_rec( $data ) {
  global $address_table;
  if( $data['addressID'] ) {
    $rows = fetch_address_rec( $data['addressID'] );
    if( $rows ) {
      $row = $rows[0];
      $query = prepare_replace_query( $address_table, $row, $data );
      return db_replace( $query );
    }
  }
  return 0;
}

function delete_address_rec( $addressID ) {
  global $address_table;
  if( $addressID ) {
    $query = "DELETE FROM $address_table WHERE addressID='$addressID'";
    return db_delete( $query );
  }
}

/*************************************************************************
ALBUMS
*************************************************************************/
function delete_album_plinks( $gedcom, $entityID) {
  global $album2entities_table;
  if( empty( $gedcom ) || empty( $entityID ) ) return false;
  $query = "DELETE FROM $album2entities_table
    WHERE gedcom='$gedcom' AND entityID='$entityID'";
  return db_delete( $query );
}

/*************************************************************************
ASSOCIATIONS
*************************************************************************/
function delete_association_records( $gedcom, $personID ) {
  global $associations_table;

  if( empty( $gedcom ) || empty( $personID ) ) return false;
  $query = "DELETE FROM $associations_table
    WHERE gedcom='$gedcom' AND (personID='$personID' OR passocID='$personID')";
  return db_delete( $query );
}

/*************************************************************************
BRANCHES
*************************************************************************/
function delete_branchlinks( $gedcom, $persfamID ) {
global $branchlinks_table;
  if( empty( $gedcom ) || empty( $persfamID ) ) return false;
  $query = "DELETE FROM $branchlinks_table
    WHERE gedcom='$gedcom' AND persfamID='$persfamID'";
  return db_delete( $query );
}

/*************************************************************************
CHILDREN
*************************************************************************/
function count_kids( $gedcom, $familyID ) {
global $children_table;
  if( !empty( $familyID ) ) {
    $query = "SELECT COUNT(*) FROM $children_table
      WHERE gedcom='$gedcom' AND familyID='$familyID'";
    return db_count( $query );
  }
  return 0;
}

function fetch_children_recs( $gedcom, $familyID ) {
global $children_table;
  $query = "SELECT * FROM $children_table
    WHERE gedcom='$gedcom' AND familyID='$familyID'
    ORDER BY ordernum";
  return db_select( $query );
}

function fetch_children( $gedcom, $familyID ) {
global $children_table, $people_table;
  $query = "SELECT *, ct.ID as chin FROM $children_table ct
    LEFT JOIN $people_table USING (gedcom, personID)
    WHERE gedcom='$gedcom'
    AND familyID='$familyID' ORDER BY ordernum";
  return db_select( $query );
}

//this function should be obsoleted -- inserts require data array as arg
function insert_child_rec( $gedcom, $personID, $familyID ) {
  global $children_table;
  if( empty( $gedcom ) || empty( $personID ) || empty( $familyID ) ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, INSERT_ERR );
    trigger_error( INSERT_ERR, E_USER_WARNING );
  }
  $data['gedcom'] = $gedcom;
  $data['personID'] = $personID;
  $data['familyID'] = $familyID;
	$data['ordernum'] = fetch_next_ordernum( $data['gedcom'], $data['familyID'] );
	$data['parentorder'] = fetch_next_parentorder( $data['gedcom'], $data['personID'] );
  $data['haskids'] = haskids( $data['gedcom'], $data['personID'] );
	$query = prepare_insert_query( $children_table, $data );
	$recno = db_insert( $query );
  if( !$recno ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, INSERT_ERR );
    trigger_error( INSERT_ERR, E_USER_WARNING );
  }
	return $recno;
}

function insert_child_record( $data ) {
  global $children_table;
	$data['ordernum'] = fetch_next_ordernum( $data['gedcom'], $data['familyID'] );
	$data['parentorder'] = fetch_next_parentorder( $data['gedcom'], $data['personID'] );
  $data['haskids'] = haskids( $data['gedcom'], $data['personID'] );
	$query = prepare_insert_query( $children_table, $data );
	$recno = db_insert( $query );
	return $recno;
}

function fetch_next_ordernum( $gedcom, $familyID ) {
global $children_table;
  $query = "SELECT MAX(ordernum) as num FROM $children_table
    WHERE gedcom='$gedcom' AND familyID='$familyID'";
  $rows = db_select( $query );
  return $rows[0]['num'] + 1;
}

function fetch_next_parentorder( $gedcom, $personID ) {
global $children_table;
  $query = "SELECT MAX(parentorder) as num FROM $children_table
    WHERE gedcom='$gedcom' AND personID='$personID'";
  $rows = db_select( $query );
  return $rows[0]['num'] + 1;
}

function fetch_next_spouse_order( $gedcom, $personID, $role ) { //VER
global $families_table;
  $spouseorder = $role == "husband" ? "husborder" : "wifeorder";
  $query = "SELECT MAX($spouseorder) as num FROM $families_table
    WHERE gedcom='$gedcom' AND $role='$personID'";
  $rows = db_select( $query );
  return $rows[0]['num'] + 1;
}

function haskids( $gedcom, $personID ) {
global $families_table, $children_table;
  $query = "SELECT familyID from $families_table
    WHERE gedcom='$gedcom' AND
    (husband='$personID' OR wife='$personID')";
  $rows = db_select( $query );
  $haskids = 0;
  foreach( $rows as $row ) {
    $query = "SELECT COUNT(*) as cnt FROM $children_table
      WHERE gedcom='$gedcom' AND familyID='{$row['familyID']}' LIMIT 1";
		$rows = db_select( $query );
    $haskids = $rows[0]['cnt'];
    if( $haskids ) break;
  }
  return $haskids ? 1 : 0;
}

function update_haskids( $gedcom, $personID ) {
global $families_table, $children_table;
  $haskids = haskids( $gedcom, $personID );
	$query = "UPDATE $children_table SET haskids='$haskids'
      WHERE gedcom='$gedcom' AND personID='$personID'";
  return db_update( $query );
}

function update_parents_haskids( $gedcom, $familyID ) {
	global $families_table;
	$query = "SELECT husband, wife FROM $families_table
    WHERE gedcom='$gedcom' AND familyID='$familyID'";
	$rows = db_select( $query );
  if( count( $rows ) ) {
    $famrow = $rows[0];
  	if( $famrow['husband'] )
  		update_haskids( $gedcom, $famrow['husband'] );
  	if( $famrow['wife'] )
  		update_haskids( $gedcom, $famrow['wife'] );
  }
}

// this should be deprecated because it uses array for args
// Principal: only use args-array for insert/update ops
function delete_child_rec( $args ) {
global $children_table;
  if( !$args['gedcom'] || !$args['familyID'] || !$args['personID'] ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, ARGS_ERR );
    trigger_error( ARGS_ERR, E_USER_WARNING );
    return 0;
  }
  $query = "DELETE FROM $children_table
    WHERE gedcom='{$args['gedcom']}'
    AND familyID='{$args['familyID']}'
    AND personID='{$args['personID']}'";
	$affrows = db_delete( $query );
  if( !$affrows ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, DELETE_ERR );
    trigger_error( DELETE_ERR, E_USER_WARNING );
  }
  // if this leaves parents childless reset haskids flags
  update_parents_haskids( $args['gedcom'], $args['familyID'] );
  return $affrows;
}

function delete_child_record( $gedcom, $familyID, $personID ) {
global $children_table;
  $query = "DELETE FROM $children_table
    WHERE gedcom='$gedcom'
    AND familyID='$familyID'
    AND personID='$personID'";
	$affrows = db_delete( $query );
  if( !$affrows ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, DELETE_ERR );
    trigger_error( DELETE_ERR, E_USER_WARNING );
  }
  return $affrows;
}

function delete_child_records( $gedcom, $personID ) {
global $children_table;
  if( empty( $gedcom ) || empty( $personID ) ) return false;
  $query = "SELECT ID, familyID FROM $children_table
    WHERE gedcom='$gedcom' AND personID='$personID'";
  $rows = db_select( $query );
  $rowcount = count( $rows );
  if( $rowcount ) {
    foreach( $rows as $row ) {
      $query2 = "DELETE FROM $children_table WHERE ID='{$row['ID']}'";
      db_delete( $query2 );
      update_parents_haskids( $gedcom, $row['familyID'] );
    }
  }
  return $rowcount;
}

function unlink_children( $gedcom, $familyID ) {
  global $children_table;
    $query = "DELETE FROM $children_table
      WHERE gedcom='$gedcom' AND familyID='$familyID'";
    return db_delete( $query );
}

/*************************************************************************
CITATIONS
*************************************************************************/
function fetch_event_citations( $gedcom, $persfamID, $eventID ) {
global $citations_table;
	$query = "SELECT * FROM $citations_table
		WHERE gedcom='$gedcom'
    AND persfamID='$persfamID'
    AND eventID='$eventID' ORDER BY ordernum";
	return db_select( $query );
}

function insert_citation( $data ) {
global $citations_table;
	$query = prepare_insert_query( $citations_table, $data );
	$recno = db_insert( $query );
  if( !$recno ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, INSERT_ERR );
    trigger_error( INSERT_ERR, E_USER_WARNING );
  }
  return $recno;
}

function update_citation( $data ) {
global $citations_table;
  $affrows = 0;
  $query = "SELECT * FROM $citations_table
    WHERE citationID = '{$data['citationID']}'";
  $rows = db_select( $query );
  if( count( $rows ) ) {
    $row = $rows[0];
    $query = prepare_replace_query( $citations_table, $row, $data );
    $affrows = db_replace( $query );
    if( !$affrows ) {
      $backtrace = debug_backtrace();
      dblog( $backtrace, REPLACE_ERR );
      trigger_error( REPLACE_ERR, E_USER_WARNING );
    }
  }
  return $affrows;
}

function delete_citation( $citationID ) {
global $citations_table;
  $query = "DELETE FROM $citations_table WHERE citationID='$citationID'";
	$affrows = db_delete( $query );
  if( !$affrows ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, DELETE_ERR );
    trigger_error( DELETE_ERR, E_USER_WARNING );
  }
  return $affrows;
}

function delete_citation_records( $gedcom, $persfamID ) {
global $citations_table;
  if( empty( $gedcom ) || empty( $persfamID ) ) return false;
  $query = "DELETE FROM $citations_table
    WHERE gedcom='$gedcom' AND persfamID='$persfamID'";
  return db_delete( $query );
}

/*************************************************************************
EVENTS
**************************************************************************/
function insert_event( $data ) {
global $events_table;

	$query = prepare_insert_query( $events_table, $data );
	$backtrace = debug_backtrace();
	dblog( $backtrace, $query );
	$recno = db_insert( $query );
	if( !$recno ) {
		$backtrace = debug_backtrace();
		dblog( $backtrace, INSERT_ERR );
		trigger_error( INSERT_ERR, E_USER_WARNING );
	}
	else
	  	return $recno;
}

function fetch_event_rec( $eventID ) {
global $events_table;
  if( $eventID ) {
    $query = "SELECT * FROM $events_table WHERE eventID='$eventID'";
    return db_select( $query );
  }
  return array();
}

function update_event_rec( $data ) {
global $events_table;
  if( $data['eventID'] ) {
    $rows = fetch_event_rec( $data['eventID'] );
    $row = $rows[0];
    $query = prepare_replace_query( $events_table, $row, $data );
    $affrows = db_replace( $query );
    if( !$affrows ) {
      $backtrace = debug_backtrace();
      dblog( $backtrace, UPDATE_ERR . " eventID {$data['eventID']}" );
      trigger_error( UPDATE_ERR . " eventID {$data['eventID']}", E_USER_WARNING );
    }
    else
      return $affrows;
  }
}

function delete_event( $eventID ) {
global $events_table;
  $query = "DELETE FROM $events_table WHERE eventID='$eventID'";
	$affrows = db_delete( $query );
  if( !$affrows ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, DELETE_ERR );
    trigger_error( DELETE_ERR, E_USER_WARNING );
  }
  else
    return $affrows;
}

function delete_event_records( $gedcom, $persfamID ) {
global $events_table;
  if( empty( $gedcom ) || empty( $persfamID ) ) return false;
  $query = "DELETE FROM $events_table
    WHERE gedcom='$gedcom' AND persfamID='$persfamID'";
  return db_delete( $query );
}

/*************************************************************************
EVENTTYPES
**************************************************************************/

function insert_eventtype ($data) {
global $eventtypes_table;
	$query = prepare_insert_query( $eventtypes_table, $data );
  $eventtypeID = db_insert( $query );
	if( !$eventtypeID ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, INSERT_ERR );
    trigger_error( INSERT_ERR, E_USER_WARNING );
  }
  return $eventtypeID;
}

/*************************************************************************
FAMILIES
*************************************************************************/

function fetch_family_rec( $gedcom, $familyID ) {
global $families_table;
  $query = "SELECT * FROM $families_table
    WHERE gedcom='$gedcom' AND familyID='$familyID'";
  $rows = db_select( $query );
  if( $rows ) return $rows[0];
  return $rows;
}

function fetch_marriage_recs( $gedcom, $personID ) {
global $families_table;
	$query = "SELECT *, 'husband' as role FROM $families_table
		WHERE gedcom='$gedcom' AND husband='$personID'
		ORDER BY husborder";
	$rows = db_select( $query );
  if( !count( $rows ) ) {
		$query = "SELECT *, 'wife' as role FROM $families_table
			WHERE gedcom='$gedcom' AND wife='$personID'
			ORDER BY wifeorder";
	  $rows = db_select( $query );
  }
	return $rows;
}

function fetch_parent_sets( $gedcom, $personID ) {
global $children_table, $families_table;
  $query = "SELECT ft.* FROM $children_table ct, $families_table ft
    WHERE ct.personID='$personID'
    AND ct.gedcom='$gedcom'
    AND ct.familyID=ft.familyID ORDER BY ct.parentorder";
  return db_select( $query );
}

function insert_family_rec( $data ) {
  global $families_table;
  $gedcom = $data['gedcom'];
  $living = $private = 0;
  $data['familyID'] = fetch_next_id( 'F', $data['gedcom'] );

  if( !empty( $data['husband'] ) ) {
    $data['husborder'] = fetch_next_husborder( $gedcom, $data['husband'] );
    $husband_flag_array = fetch_person_flag_array( $gedcom, $data['husband'] );
    $living = $husband_flag_array['living'] ? $husband_flag_array['living'] : 0;
    $private = $husband_flag_array['private'] ? $husband_flag_array['private'] : 0;;
  }
  else
    $data['husborder'] = 0;

  if( !empty( $data['wife'] ) ) {
    $data['wifeorder'] = fetch_next_wifeorder( $gedcom, $data['wife'] );
    $wife_flag_array = fetch_person_flag_array( $gedcom, $data['wife'] );
    $living = $wife_flag_array['living'] ? $wife_flag_array['living'] : 0;
    $private = $wife_flag_array['private'] ? $wife_flag_array['private'] : 0;
  }
  else
    $data['wifeorder'] = 0;

	$query = prepare_insert_query( $families_table, $data );
  $newID = db_insert( $query );
  if( !$newID ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, INSERT_ERR );
    trigger_error( INSERT_ERR, E_USER_WARNING );
  }
	return $data['familyID'];
}

//this is klunky and shoulde be deprecated
function insert_family_record( $data ) {
global $families_table;
  $gedcom = $data['gedcom'];
  $role = $data['role'];
  $parentID = $data[$role];
  $data['living'] = fetch_living_flag( $gedcom, $parentID );
  $data['familyID'] = fetch_next_id( 'F', $data['gedcom'] );


  if( !empty( $data['husband'] ) )
    $data['husborder'] = fetch_next_husborder( $gedcom, $data['husband'] );
  else
    $data['husborder'] = 0;

  if( !empty( $data['wife'] ) )
    $data['wifeorder'] = fetch_next_wifeorder( $gedcom, $data['wife'] );
  else
    $data['wifeorder'] = 0;

	$query = prepare_insert_query( $families_table, $data );
  $newID = db_insert( $query );
  if( !$newID ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, INSERT_ERR );
    trigger_error( INSERT_ERR, E_USER_WARNING );
  }
	return $data['familyID'];
}

function fetch_next_husborder( $gedcom, $husbandID ) {
global $families_table;
  if( empty( $husbandID ) ) return 0;
  $query = "SELECT MAX(husborder) as husborder FROM $families_table
    WHERE gedcom='$gedcom' AND husband='$husbandID'";
  $rows = db_select( $query );
  return $rows[0]['husborder'] + 1;
}

function fetch_next_wifeorder( $gedcom, $wifeID ) {
global $families_table;
  if( empty( $wifeID ) ) return 0;
  $query = "SELECT MAX(wifeorder) as wifeorder FROM $families_table
    WHERE gedcom='$gedcom' AND wife='$wifeID'";
  $rows = db_select( $query );
  return $rows[0]['wifeorder'] + 1;
}

function update_family_record($data) {
global $families_table;
	if( empty( $data['gedcom'] ) || empty( $data['familyID'] ) ) {
    $backtrace = debug_backtrace();
		dblog( $backtrace, ARGS_ERR );
    trigger_error( ARGS_ERR, E_USER_WARNING );
    exit;
	}
  $query = "SELECT * FROM $families_table
    WHERE gedcom='{$data['gedcom']}' AND familyID='{$data['familyID']}'";
  $rows = db_select( $query );
  if( !count( $rows ) ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, NOREC_ERR );
    trigger_error( NOREC_ERR, E_USER_WARNING );
    exit;
  }
  $row = $rows[0];
  $query = prepare_replace_query( $families_table, $row, $data );
  $affrows = db_replace( $query );

  if( !$affrows ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, REPLACE_ERR );
    trigger_error( REPLACE_ERR, E_USER_WARNING );
  }
	return $affrows;
}

//klunky! deprecate this one
function unlink_spouse( $gedcom, $familyID, $role ) {
global $families_table;
  $order = $role == "husband" ? "husborder" : "wifeorder";
  $query = "UPDATE $families_table SET $role='', $order=''
    WHERE gedcom='$gedcom' AND familyID='$familyID'";
	$affrows = db_update( $query );
  if( !$affrows ){
    $backtrace = debug_backtrace();
    dblog( $backtrace, "Unable to remove $role from family record $familyID!" );
    trigger_error( "Unable to remove $role from family record $familyID" );
  }
  return $affrows;
}

function remove_spouse( $gedcom, $familyID, $personID ) {
global $families_table;
  if( empty( $gedcom ) || empty( $familyID ) || empty( $personID ) ) return false;
  $query = "SELECT * FROM $families_table
    WHERE gedcom='$gedcom'
    AND familyID='$familyID'
    AND ( husband='$personID' or wife='$personID')";
  $rows = db_select( $query );
  $rowcount = count( $rows );
  if( $rowcount ) {
    $row = $rows[0];
    $kids = count_kids( $gedcom, $familyID );
    if( $row['husband'] == $personID ) {
      //don't leave orphaned marriage record
      if( empty( $row['wife'] ) && $kids == 0 ) {
        $query = "DELETE FROM $families_table WHERE ID='{$row['ID']}'";
        db_delete( $query );
        $familyID = '';
      }
      else {
        $query = "UPDATE $families_table SET husband='', husborder=''
          WHERE ID='{$row['ID']}'";
          db_update( $query );
      }
    }
    else { // personID must be wife
      //don't leave orphaned marriage record
      if( empty( $row['husband'] ) && $kids == 0 ) {
        $query = "DELETE FROM $families_table WHERE ID='{$row['ID']}'";
        db_delete( $query );
        $familyID = '';
      }
      else {
        $query = "UPDATE $families_table SET wife='', wifeorder=''
          WHERE ID='{$row['ID']}'";
        db_update( $query );
      }
    }
  }
  return $familyID;
}

function remove_spouse_from_all( $gedcom, $personID ) {
global $families_table;
  if( empty( $gedcom ) || empty( $personID ) ) return false;
  $query = "SELECT * FROM $families_table
    WHERE gedcom='$gedcom'
    AND ( husband='$personID' or wife='$personID')";
  $rows = db_select( $query );
  $rowcount = count( $rows );
  if( $rowcount ) {
    foreach( $rows as $row ) {
      $kids = count_kids( $gedcom, $row['familyID'] );
      if( $row['husband'] == $personID ) {
        //don't leave orphaned marriage records
        if( empty( $row['wife'] ) && $kids == 0 ) {
          $query = "DELETE FROM $families_table WHERE ID='{$row['ID']}'";
          db_delete( $query );
        }
        else {
          $query = "UPDATE $families_table husband='', husborder=''
            WHERE ID='{$row['ID']}'";
            db_update( $query );
        }
      }
      else { // personID must be wife
        //don't leave orphaned marriage records
        if( empty( $row['husband'] ) && $kids == 0 ) {
          $query = "DELETE FROM $families_table WHERE ID='{$row['ID']}'";
          db_delete( $query );
        }
        else {
          $query = "UPDATE $families_table wife='', wifeorder=''
            WHERE ID='{$row['ID']}'";
          db_update( $query );
        }
      }
    }
  }
  return $rowcount;
}

function link_spouse( $gedcom, $familyID, $role, $personID ){
global $families_table;
  $spouseorder = $role == "husband" ? "husborder" : "wifeorder";
  $ordernum = fetch_next_spouse_order( $gedcom, $personID , $role );
  $query = "UPDATE $families_table SET $role='$personID', $spouseorder='$ordernum'
    WHERE gedcom='$gedcom' and familyID='$familyID'";
	$affrows = db_update( $query );
  if( !$affrows ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, UPDATE_ERR );
    trigger_error( UPDATE_ERR, E_USER_WARNING );
  }
  return $affrows;
}

function set_family_rec_living( $gedcom, $familyID ) {
global $families_table, $people_table;
  $query = "SELECT * FROM $families_table
    WHERE gedcom='$gedcom' AND familyID='$familyID'";
  $rows = db_select( $query );
  if( !count( $rows ) ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, NOREC_ERR );
    trigger_error( NOREC_ERR, E_USER_WARNING );
  }
  else {
    $row = $rows[0];
    $living = fetch_living_flag( $gedcom, $row['husband'] );
    if( $living == 0 ) {
      $living = fetch_living_flag( $gedcom, $row['wife'] );
    }
    $query = "UPDATE $families_table SET living='$living'
      WHERE gedcom='$gedcom' AND familyID='$familyID'";
    return db_update( $query );
  }
}

function delete_family_record( $gedcom, $familyID ) {
global $families_table;
  if( empty( $gedcom ) || empty( $familyID ) ) return false;
  $query = "DELETE FROM $families_table
    WHERE gedcom='$gedcom' AND familyID='$familyID'";
  return db_delete( $query );
}

function delete_family_full( $gedcom, $familyID ){
  if( !delete_family_record( $gedcom, $familyID ) ) return false;
  unlink_children( $gedcom, $familyID );
  delete_album_plinks( $gedcom, $familyID );
  delete_branchlinks( $gedcom, $familyID );
  delete_citation_records( $gedcom, $familyID );
  delete_event_records( $gedcom, $familyID );
  delete_medialinks( $gedcom, $familyID );
  delete_note_records( $gedcom, $familyID );
}

/*************************************************************************
MEDIA
*************************************************************************/
function delete_medialinks( $gedcom, $persfamID ) {
  global $medialinks_table;
  if( empty( $gedcom ) || empty( $persfamID ) ) return false;
  $query = "DELETE FROM $medialinks_table
    WHERE gedcom='$gedcom' AND personID='$persfamID'";
  return db_delete( $query );
}

/*************************************************************************
MOSTWANTED
*************************************************************************/
function delete_mostwanted_records( $gedcom, $personID ) {
global $mostwanted_table;
  if( empty( $gedcom ) || empty( $personID ) ) return false;
  $query = "DELETE FROM $mostwanted_table
    WHERE gedcom='$gedcom' AND personID='$personID'";
  return db_delete( $query );
}

/*************************************************************************
NOTELINKS AND XNOTES
*************************************************************************/
function insert_note( $data ) {
global $xnotes_table, $notelinks_table;
  $data['secret'] = empty( $data['secret'] ) ? 0: 1;
  $query = prepare_insert_query( $xnotes_table, $data );
  $data['xnoteID'] = db_insert( $query );

  $query = prepare_insert_query( $notelinks_table, $data );
  $recno = db_insert( $query );
  if( !$recno ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, INSERT_ERR );
    trigger_error( INSERT_ERR, E_USER_WARNING );
  }
  return $recno;
}

function update_note( $data ) {
global $xnotes_table, $notelinks_table;
	$xnoteID = $data['xnoteID'];
	$gedcom = $data['gedcom'];
	$data['secret'] = $data['secret'] == 'on' ? 1: 0;
  $query = "SELECT * FROM $xnotes_table
    WHERE gedcom='$gedcom' AND ID='$xnoteID'";
  $rows = db_select( $query );
  $row = $rows[0];
  $query = prepare_replace_query( $xnotes_table, $row, $data );
  $affrows = db_replace( $query );
  if( !$affrows ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, REPLACE_ERR );
    trigger_error( REPLACE_ERR, E_USER_WARNING );
  }
  $query = "SELECT * FROM $notelinks_table
    WHERE gedcom='$gedcom' AND xnoteID='$xnoteID'";
  $rows = db_select( $query );
  $row = $rows[0];
  $query = prepare_replace_query( $notelinks_table, $row, $data );
  $affrows = db_replace( $query );
  if( !$affrows ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, REPLACE_ERR );
    trigger_error( REPLACE_ERR, E_USER_WARNING );
  }
  return $affrows;
}

function delete_note( $xnoteID ) {
global $citations_table, $notelinks_table, $xnotes_table;
  // may or may not be an associated citation on the note
	$src_eventID = "N$xnoteID";
  $query = "DELETE FROM $citations_table WHERE eventID='$src_eventID'";
  db_delete( $query );

  $query = "DELETE FROM $notelinks_table WHERE xnoteID='$xnoteID'";
  $affrows = db_delete( $query );
   if( !$affrows ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, DELETE_ERR );
    trigger_error( DELETE_ERR, E_USER_WARNING );
  }
  $query = "DELETE FROM $xnotes_table WHERE ID='$xnoteID'";
  $affrows = db_delete( $query );
  if( !$affrows ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, DELETE_ERR );
    trigger_error( DELETE_ERR, E_USER_WARNING );
  }
  return $affrows;
}

function delete_note_records( $gedcom, $persfamiID ) {
global $notelinks_table;
  if( empty( $gedcom ) || empty( $persfamID ) ) return false;
  $query = "SELECT xnoteID FROM $notelinks_table
    WHERE gedcom='$gedcm' AND persfamID='$persfamID'";
  $rows = db_select( $query );
  $rowcount = count( $rows );
  if( $rowcount ) {
    foreach($rows as $row ) {
      delete_note( $row['xnoteID'] );
    }
  }
  return $rowcount;
}

/*************************************************************************
PEOPLE
*************************************************************************/
function fetch_person_flag_array( $gedcom, $personID ) {
global $people_table;
  $query = "SELECT living, private FROM $people_table
    WHERE gedcom='$gedcom' AND personID='$personID'";
  $rows = db_select( $query );
  $rowcount = count( $rows );
  if( $rowcount ) {
    return $rows[0];
  }
  return array();
}

function fetch_living_flag( $gedcom, $personID ) {
global $people_table;
  $query = "SELECT living FROM $people_table
  WHERE gedcom='$gedcom' AND personID='$personID'";
  $rows = db_select( $query );
  if( count( $rows ) ) return $rows[0]['living'];
  return 0;
}

function fetch_name_full( $gedcom, $personID ) {
global $people_table;
  $query = "SELECT firstname, lastname, nickname, title, prefix, suffix
    FROM $people_table
    WHERE gedcom='$gedcom' AND personID='$personID'";
  $rows = db_select( $query );
  if( !count( $rows ) ) return "";
  $nameset = $rows[0];
	return format_name( $nameset, $full=true );
}

function fetch_name_simple( $gedcom, $personID ) {
global $people_table;
  $query = "SELECT firstname, lastname FROM $people_table
    WHERE gedcom='$gedcom' AND personID='$personID'";
  $rows = db_select( $query );
  $row = count( $rows ) ? $rows[0] : array();
	return format_name( $row );
}

function format_name( $nameset, $full=false ) {
  if( empty( $nameset['firstname'] ) && empty( $nameset['lastname'] ) )
    return "[unknown]";
	$firstname = !empty( $nameset['firstname'] ) ? trim( $nameset['firstname'] ) : NULL;
	$lastname = !empty( $nameset['lastname'] ) ? trim( $nameset['lastname'] ) : NULL;
	if( $full ) {
	$nickname = !empty( $nameset['nickname'] ) ? trim( $nameset['nickname'] ) : NULL;
	$title = !empty( $nameset['title'] ) ? trim( $nameset['title'] ) : NULL;
	$prefix = !empty( $nameset['prefix'] ) ? trim( $nameset['prefix'] ) : NULL;
	$suffix = !empty( $nameset['suffix'] ) ? trim( $nameset['suffix'] ) : NULL;
	}
	$namestr = "";
	$namestr .= empty( $title ) ? '' : trim( $title ) . " ";
	$namestr .= empty( $firstname ) ? "[unknown] " : trim( $firstname ) . " ";
	$namestr .= empty( $nickname ) ? '' : "(" . trim( $nickname ) . ") ";
	$namestr .= empty( $prefix ) ? '' : trim( $prefix ) . " ";
	$namestr .= empty( $lastname ) ? "[unknown]" : trim( $lastname );
	$namestr .= empty( $suffix ) ? '' : ", " . trim( $suffix );
	return $namestr;
}

function fetch_person( $gedcom, $personID ) {
  global $people_table;
  $query = "SELECT * FROM $people_table
    WHERE gedcom='$gedcom' AND personID='$personID'";
  $rows = db_select( $query );
  if( $rows ) return $rows[0];
  return $rows;
}

function insert_person( $data ) {
global $people_table, $currentuser, $options;
  $data['personID'] = fetch_next_id( $options['pid_prefix'], $data['gedcom'] );
  $data['metaphone'] = metaphone($data['lastname']);
  $data['living'] = isset( $data['living'] ) ? 1 : 0;
  $data['private'] = isset( $data['private'] ) ? 1 : 0;
  $query = prepare_insert_query( $people_table, $data);
  $record_id = db_insert( $query );
  if( !$record_id ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, INSERT_ERR );
    trigger_error( INSERT_ERR, E_USER_WARNING );
  }
	return $data['personID'];
}

function set_person_famc( $gedcom, $personID ) {
global $children_table, $people_table;
  // get list of existing families (famc ) for subject and select first one
  $query = "SELECT familyID FROM $children_table
    WHERE gedcom='$gedcom' AND personID='$personID'
    ORDER BY ID";
  $rows = db_select( $query );
  $count = count( $rows );
  $famclist = array();
  foreach( $rows as $row ) {
    $famclist[] = $row['familyID'];
  }
  if( !empty( $famclist ) ) {
    // look at current famc
    $query = "SELECT famc FROM $people_table
      WHERE gedcom='$gedcom' AND personID='$personID'";
    $rows = db_select( $query );
    $famc = $rows[0]['famc'];
    // if famc is still good return without doing anything
    if( in_array( $famc, $famclist ) ) {
      return 0;
    }
    // otherwise assign the first family in the families lis
    else {
      $famc = $famclist[0];
    }
  }
  // or there are no families so remove the famc from person record
  else {
    $famc = '';
  }
  $query = "UPDATE $people_table SET famc='$famc'
    WHERE gedcom='$gedcom' AND personID='$personID'";
  return db_update( $query );
}

function sex_selector( $selected, $disabled='' ) {
	$selectM = $selectF = $selectU = '';
	if (!empty ($disabled)) $disabled = 'DISABLED';
	if ($selected == 'M') $selectM = 'selected';
	else if ($selected == 'F') $selectF = 'selected';
	else $selectU = 'selected';
	$outstring = "
    <select id='sex' name ='sex' $disabled>
      <option $selectM value='M'>M</option>
      <option $selectF value='F'>F</option>
      <option $selectU value='U'>U</option>
    </select>
";
	return $outstring;
}

function ret_name_string( $person ) {
	return format_name( $person );
}

function update_person( $data ) {
global $people_table, $currentuser;
  if( empty( $data['gedcom'] ) || empty( $data['personID'] ) ) return 0;
  $query = "SELECT * FROM $people_table
    WHERE gedcom='{$data['gedcom']}' AND personID='{$data['personID']}'";
  $rows = db_select( $query );
  if( !count( $rows ) ) return 0;
  $person = $rows[0];
  $data['living'] = isset( $data['living'] ) ? 1 : 0;
  $data['private'] = isset( $data['private'] ) ? 1 : 0;
  $query = prepare_replace_query( $people_table, $person, $data );
  $affrows = db_replace( $query );
  if( !$affrows ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, REPLACE_ERR );
    trigger_error( REPLACE_ERR, E_USER_WARNING );
  }
  return $affrows;
}

function delete_person( $gedcom, $personID ){
global $people_table;
  if( empty( $gedcom ) || empty( $personID ) ) return false;
  $query = "DELETE FROM $people_table
    WHERE gedcom='$gedcom' AND personID='$personID'";
    return db_delete( $query );
}

function delete_person_full( $gedcom, $personID ){
  if( empty( $gedcom ) || empty( $personID ) ) return false;
  if( !delete_person( $gedcom, $personID ) ) return false;
  delete_child_records( $gedcom, $personID );
  unlink_spouses_all( $gedcom, $personID );
  delete_event_records( $gedcom, $personID );
  delete_citation_records( $gedcom, $personID );
  delete_album_plinks( $gedcom, $personID );
  delete_association_records( $gedcom, $personID );
  delete_branchlinks( $gedcom, $personID );
  delete_medialinks( $gedcom, $personID );
  delete_mostwanted_records( $gedcom, $personID );
  delete_note_records( $gedcom, $persfamID );
}

/*************************************************************************
PLACES
*************************************************************************/
function fetch_placename($gedcom, $placename) {
global $places_table, $tngconfig;
  $placename = db_escape( $placename );
  $gedcomstr = $tngconfig['places1tree'] == 0 ? "AND gedcom='$gedcom'" : null;
  $query = "SELECT * FROM $places_table
    WHERE place='$placename' $gedcomstr";
  $rows = db_select( $query );
  if( !count( $rows ) ) return '';
  return $rows[0]['place'];
}

function insert_placename( $data ) {
global $places_table, $tngconfig;
  if( $tngconfig['places1tree'] == 1 ) $data['gedcom'] = '';
	$query = prepare_insert_query( $places_table, $data );
	$newID = db_insert( $query );
  if( !$newID ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, INSERT_ERR );
    trigger_error( INSERT_ERR, E_USER_WARNING );
  }
  return $newID;
}

/*************************************************************************
SOURCES
*************************************************************************/
function fetch_source( $gedcom, $sourceID ) {
  global $sources_table;
	$query = "SELECT * FROM $sources_table
		WHERE gedcom='$gedcom' AND sourceID='$sourceID'";
  $rows = db_select( $query );
  if( $rows ) return $rows[0];
  return $rows;
}

function insert_source( $data ) {
  global $sources_table;
  $data['sourceID'] = fetch_next_id( 'S', $data['gedcom'] );
  $query = prepare_insert_query( $sources_table, $data);
  $record_id = db_insert( $query );
  if( !$record_id ) {
    $backtrace = debug_backtrace();
    dblog( $backtrace, INSERT_ERR );
    trigger_error( INSERT_ERR, E_USER_WARNING );
  }
	return $data['sourceID'];
}

function delete_source( $gedcom, $sourceID ) {
  global $sources_table, $citations_table;
  $query = "DELETE FROM $sources_table
    WHERE gedcom='$gedcom' AND sourceID='$sourceID'";
  $affrows = db_delete( $query );
  if( $affrows ) {
    $query = "DELETE FROM $citations_table
      WHERE gedcom='$gedcom' AND sourceID='$sourceID'";
  }
  return $affrows;
}

/*************************************************************************
TREES
**************************************************************************/
function fetch_first_gedcom() {
  global $trees_table;
  $query = "SELECT gedcom FROM $trees_table ORDER BY treename LIMIT 1";
  $rows = db_select( $query );
  return $rows[0]['gedcom'];
}

function fetch_gedcom_names() {
  global $trees_table;
  $query = "SELECT gedcom, treename FROM $trees_table ORDER BY treename";
  return db_select( $query );
}

function ret_tree_options( $gedcom="" ) {
  global $trees_table;
  $trees = fetch_gedcom_names();
  if( empty( $gedcom ) ) {
    $gedcom = $trees[0]['gedcom'];
    set_session_var( "gedcom", $gedcom );
  }
  $treestring  = "
<select id='gedcom' name ='gedcom' >";
  foreach ($trees as $tree) {
    $selected = $gedcom == $tree['gedcom'] ? "selected" : "";
    $treestring .= "
  <option $selected value='{$tree['gedcom']}' >{$tree['treename']}</option>";
  }
  $treestring .= "
</select>";
  return $treestring;
}

?>
