<?php
/*
	Needed by all primary FGW scripts to:
	1) determine relative path from FGW to TNG
	2) make both TNG and FGW configuration values available to FGW
	3) provide FGW language support to both visitors and FGW admins
	4) add sensible error reporting for all FGW scripts
*/

/*
 * Check after including language support to see if someone is 
 * accessing this script directly (must only be included).
 * If included value will be greater than 1.
*/
$access_status = count(get_included_files());
                                                   
// Load user configs as $options array
require_once 'fgw_userconfig.php';
require_once 'fgw_version.php';

$splits = explode( ".", $fgw_version );
$fgw_series = $splits[0];
$mod_support = $modspath.'/fgw_'.$fgw_series;
$img = $mod_support.'/img';

// VISITOR LANGUAGE FILE
$fgw_language_file = $mylanguage.'/fgw_text.php';

// VISITOR HELP FILE
$fgw_help_file = $mylanguage.'/fgw_help.php';

// ADMINISTRATOR LANGUAGE FILE FOR CONSISTENT LOG ENTRIES
$admin_language = !empty( $language ) ? $language : 'English-UTF8';
$admin_language_file = $languages_path.'/'.$admin_language.'/fgw_text.php';

// SET MISSING LANGUAGE FILE DEFAULTS
if( !file_exists( $fgw_language_file ) ) 
{
	$fgw_language_file = $languages_path.'English-UTF8/fgw_text.php';
}
if( !file_exists( $fgw_help_file ) ) 
{
	$fgw_help_file = $languages_path.'English-UTF8/fgw_help.php';
}
if( !file_exists( $admin_language_file ) ) 
{
	$admin_language_file = $languages_path.'/English-UTF8/fgw_text.php';
}
if( empty( $fgw_language_file ) ) 
{
	trigger_error( 'Language support error', E_USER_ERROR );
	exit;
}

// SUPPORT IN USER'S SELECTED LANGUAGE
if( !file_exists( $fgw_language_file ) ) 
{
	trigger_error('Language support missing', E_USER_ERROR );
	exit;
}
require_once $fgw_language_file;

// SUPPORT IN ADMIN'S PREFERRED LANGUAGE (FOR LOGS)
$samelanguage = $admin_language_file == $fgw_language_file ? 1 : 0;
if($samelanguage) 
{
	/* if admin == user language, set the $admtext arrary == the $text array */
	$admtext = $text;
}
else 
{
	/* otherwise get the admin language into $admtext array */
	/* two different languages */
	$admtext = get_admin_language( $admin_language_file );
}

// CHECK FOR ILLEGAL ACCESS TO THIS SCRIPT
if( $access_status == 1 ) 
{
	trigger_error( $admtext['wse_access'], E_USER_ERROR );
   die();
}

if(empty($cms['support']) && empty( $cms['tngpath'] ))
	$cms['tngpath'] = $tngpath; // only works HERE with TNGv12 +

// FULL PHP SERVER PATHS
set_include_path(get_include_path() . PATH_SEPARATOR . $tngpath);
define( 'FGWDIR', dirname(__FILE__).DIRECTORY_SEPARATOR );
define( 'TNGDIR', $rootpath );

/***************************************************************
	FUNCTIONS
***************************************************************/
/**
Keep administrators language support separate from user's
By giving the text elements a different array name (admtext)
@admin_language_file	- path to FGW support file in admin's language
*/
function get_admin_language($admin_language_file) 
{
	global $tngdomain, $sitename;
	require $admin_language_file;
	foreach( $text as $key => $value ) 
	{
		$admtext[$key] = $value;
	}
	return $admtext;
}