/*
	fgw_lib.js (43)
	JAVASCRIPT (JQUERY) ROUTINES FOR THE FAMILY GROUP WORKSHEET

	v43 allows admins to save worksheets even if no changes/additions were made
*/

/*
 *	var datastate contains the serialized data from the initial worksheet.
 * When the worksheet is submitted it is serialized again in a state variable
 * and compared with datastate to see if the user has changed the form in
 * any way.
 */
var datastate;

jQuery(function() {

	/* To check if submitted form is dirty we must record its initial the
		datastate by serializing the data inside div #fgwfgw of the clean form.
		When a form is submitted the same div #fgwfgw) is
		re-serialized and compared to the datastate to see if they differ,
		meaning changes were made -- the form is dirty.
	*/
	datastate = jQuery('#fgwfgw').find('select, textarea, input').serialize();
	//alert( datastate );

	// bot trap
	jQuery('#comments').hide();

	// This function is run when a worksheet is submitted
	jQuery('#worksheet').submit(function(e) {
		/* on submission
		 *   1) If security code is required, see that it has four characters
		 *   2) Verify:
		 			a. the form either contains new information (is dirty) OR
					b. it is submitted by an admin
		 *	  3) Serialize the form
		 *	  4) Ajax it to fgw_ajax_proc.php for for further processing
		 *
		 *   Other verification checks are made in fgw_ajax_proc.php
		 */

		e.preventDefault();

		var caplen = 0;
      // IF SECURITY CODE IS REQUIRED
		if(jQuery('#fgw_hash').length && jQuery('#fgw_hash').val().length) {
			// check for correct security code length
			var caplen = jQuery.trim(jQuery('#capcode').val()).length;
			if(caplen !=4) {
				alert( '24: ' + jQuery('#nocode').html());
				return;
			}
		}

		// CHECK #fgwfgw div (excludes security code) FOR CHANGES
		var superuser = jQuery('#superuser').html();
		var state = jQuery('#fgwfgw').find('select, textarea, input').serialize();

		// check for dirty form - give admin a pass
		if(superuser == 0 ) {
			if( datastate == state ) {
				alert('40 :' + jQuery('#nochanges').html());
				return;
			}
		}

		var sdata = jQuery('#worksheet').serialize();

		jQuery('#submitted').attr("disabled", "disabled");
		jQuery.ajax({
			type: 'POST',
			url: 'fgw_ajax_proc.php',
			data: sdata,
			beforeSend:function(){
				jQuery('#wait').show();
			},
			success: function(data){
				jQuery('#wait').hide();
				datastate = jQuery('#fgwfgw').find('select, textarea, input').serialize();
				var parts = data.split('|');
				// three part message means the form has been processed successfully
				if(parts.length==3) {
					var formID = parts[0];
					var fkey = parts[1];
					var msg = parts[2];
					jQuery('#wait').hide();
					alert('88-' + msg);
					// refresh the screen to pick up the new id and fkey
				   location.href='fgw.php';
					return;
				}
				// one-part message means there was a problem processing the form
				// so stay on the form to allow user to make corrections
				else {
					jQuery('#wait').hide();
					alert('98-' + data);
					jQuery('#submitted').removeAttr( "disabled" );
				}
			},
			complete:function() {
				jQuery('#wait').hide();
			}
		});
  });

  jQuery('#f_admin').submit(function(e) {
    e.preventDefault();
    var adata = jQuery('#f_admin').serialize();
    jQuery.ajax({
        type:'POST',
        url:'fgw_ajax_admin.php',
        data:adata,
        beforeSend:function(){
            jQuery('#wait').show();
        },
        success:function(data){
            jQuery('#wait').hide();
            alert(data);
            jQuery('#wait').show();
            location.reload(true);
        }
    });
  });

  jQuery('#gedcom').change(function(current){
    var newtree = jQuery('#gedcom :selected').val();
  	if(verify_tree()) {
  		location.href = 'fgw.php?tree=' + newtree + '&amp;persfamID=';
    }
    else {
      // set current tree back to selected
      jQuery('#gedcom option:selected').prop('selected',false);
      jQuery('#gedcom option[value='+current+']').prop('selected',true);
    }
  });

  jQuery(document).on('click','#hinsert, #winsert', function(){
    var role = (jQuery(this).attr('id')=='hinsert') ? 'husb' : 'wife';
    var spouseID = (role=='husb') ? jQuery('#husbID').val() : jQuery('#wifeID').val();
    if( spouseID.length!=0){
        getDatapack(spouseID, role);
    }
  });

	jQuery(document).on('click','#hremove, #wremove',function(){
		var role = (jQuery(this).attr('id')=='hremove') ? 'husb' : 'wife';
		var other = role == 'husb' ? '#wifeID' : '#husbID';
		var otherID = jQuery(other).val();
		var section = '#' + role + 'data';
		clear_spouse(section);
		clear_spouse('#marriage');
		set_form_title();	
	});

	//datastate = jQuery('#fgwfgw').find('select, textarea, input').serialize();
	window.onbeforeunload=function(){
		var msg='Your worksheet data has not been saved!';
		if( datastate != jQuery('#fgwfgw').find('select, textarea, input').serialize())
			return msg;
	}

   jQuery('#sortable').sortable({
      items:"table",
      handle:".handle",
      cursor:"move",
      update:function( event, ui ) {
         var order = jQuery(this).sortable('toArray');
         jQuery('#order').val(order);
         var i=1;
         jQuery('#sortable span').each(function(index, element) {
            jQuery(this).text(i);
            i++;
         })
      }
   });

  jQuery('#copyLink').click(function(){
     var one = jQuery('#copyLink').text();
     alert(one);
   });

}); // end of document ready

function reject_new(ele) {
   var table_id = jQuery(ele).closest("table").attr("id");
   //alert(table_id); return false;
   jQuery("#"+table_id+" td input.left").prop('checked', true);
   jQuery("#"+table_id+" td input.right").removeProp('checked');
   return false;
}

function clear_spouse( ele ) {
	jQuery(ele).find(':input').each(function() {
		switch(this.type) {
			case 'hidden':
			case 'password':
			//case 'select-multiple':
			//case 'select-one':
			case 'text':
			case 'textarea':
				 $(this).val('');
				 break;
			case 'checkbox':
			case 'radio':
				 this.checked = false;
		}
	});
	jQuery(ele + ' .marrs')
	.find('option')
	.remove()
	.end()
	.append( '<option value="">--None--</option>');

	jQuery('.marrs option:selected').removeAttr('selected');
	
	clearChildren();	
}

function set_form_title() {
	var title = '';
	var husbfirst = jQuery('#husbfirst').val();
	var husblast = jQuery('#husblast').val();
	var wifefirst = jQuery('#wifefirst').val();
	var wifelast = jQuery('#wifelast').val();
	var his = '';
	var hers = '';
	
	if(husbfirst.length!=0)
		his = husbfirst;
	if(husblast.length!=0)
		if(his.length!=0)
			his += ' ' + husblast;
		else
			his = husblast;
	
	if(wifefirst.length!=0)
		hers = wifefirst;
	if(wifelast.length!=0)
		if( hers.length!=0)
			hers += ' ' + wifelast;
		else
			hers = wifelast;
		
	if(his.length!=0 && hers.length!=0)
		title = his + ' & ' + hers;
	else if (his.length!=0)
		title = his;
	else
		title = hers;
	jQuery('#formTitle').html(title);
}

function wsFromChild( childNr ){
    jQuery('#wait').show();
    var data = jQuery('#worksheet').serialize();
    jQuery.post('fgw_child.php', {data:data,childNr:childNr},
    function(msg){
        jQuery('#wait').hide();
        //alert(msg);
        window.open('fgw.php?sv=1','_blank');
    });
}

function show_help( help_text ) {
    alert( help_text );
}

function verify_tree() {
  var response = confirm( jQuery('#newtree').html());
  if( response == true) {
        datastate = jQuery('#fgwfgw').find('select, textarea, input').serialize();
  }
  return response;
}


function getDatapack(personID,role) {
  var gedcom = jQuery('#gedcom').val();
  jQuery('#wait').show();
  jQuery.ajax({
		type:'POST',
		url:'fgw_ajax_spouse.php',
		data:{gedcom:gedcom,personID:personID,role:role},
		success:function(data){
			jQuery('#wait').hide();
				if( data.length > 1 ) {
				if(role == 'husb') {
				  jQuery('#husbdata').html(data);
				}
				else {
				  jQuery('#wifedata').html(data);
				}
				set_form_title();
				datastate = jQuery('#fgwfgw').find('select, textarea, input').serialize();
			}
			else {
				alert( 'personID ' + personID + ' does not exist in database!');
				jQuery('#'+role+'ID').val('');
			}
		}
	});
}

function getFamily(persfamID) {
   var gedcom = jQuery('#gedcom').val();

   jQuery('#wait').show();
   jQuery.post( 'fgw_ajax_family.php', {gedcom:gedcom,persfamID:persfamID},
      function(data){
         jQuery('#wait').hide();
         jQuery('#fgwfamily').html(data);
			datastate = jQuery('#fgwfgw').find('select, textarea, input').serialize();
  });
}

function getMarriage( familyID ) {
   var gedcom = jQuery('#gedcom').val();
   location.href="fgw.worksheet.php?persfamID="+familyID+"&amp;tree="+gedcom;
}

function addRow() {
    // get current last row in table
    var firstrow = 1; // skip header rows
    var rows = jQuery('#children .formrow');
    var idx = rows.length - firstrow;
    var plus ="<img src=\"img/plus.png\" alt=\"\" title=\""+newform+"\" onClick=\"return wsFromChild("+idx+");\"/>";
    var row = jQuery('#children .formrow:last' );
    var firstname = jQuery('input:first', row).val();
    if( !firstname ) {
         alert(jQuery('#childline').html());
         return;
    }
    else {
        jQuery('#children .formrow:last .pagecell:last').html(plus)
    }

  jQuery('#children .formrow:last').clone(true).insertAfter('#children .formrow:last');
  rows++; idx++;
  var rows = jQuery('#children .formrow');
  var idx = rows.length - firstrow;
  var num = idx+1;
  jQuery('#children .formrow:last input').each(function(i,e){
    e.name = e.name.replace(/\d/g, (idx) );
    if( !jQuery(this).is('.lastname') && !jQuery(this).is('.gedcom') ) {
      e.value = '';
    }
  });
  jQuery('#children .formrow:last select').each(function(i,e){
    e.name = e.name.replace(/\d/g, (idx) );
    e.value = 'U';
  });
  jQuery('#children .formrow:last :checkbox').each(function(i,e) {
    e.name = e.name.replace(/\d/g, (idx) );
    jQuery(this).attr('checked', true);
    jQuery(this).val('1');
  });
  jQuery('#children .formrow:last .pagecell:last').html('&nbsp;');
  jQuery('#children .formrow:last .formcell:first').html(num);
}

function clearChildren() {
	var rows = jQuery('#children .formrow');
	var numrows = rows.length;
	while ( numrows > 1 ) {
		jQuery('#children .formrow:last').remove();
		numrows--;
	}
	clearLastChildRow();
}

function deleteLastRow() {
    var firstrow = 1;
    var rows = jQuery('#children .formrow');
    //alert( 'rows: '+rows.length );exit;
    if( rows.length > firstrow ) {
        jQuery('#children .formrow:last').remove();
    }
    else {
        clearLastChildRow();
    }
}
function clearLastChildRow() {
  jQuery('#children .formrow:last input').each(function(i,e){
    if( !jQuery(this).is('.kidlastname') ) {
      e.value = '';
    }
  });
  jQuery('#children .formrow:last select').each(function(i,e){
    e.value = 'U';
  });
  jQuery('#children .formrow:last :checkbox').each(function(i,e) {
    jQuery(this).attr('checked', true);
    jQuery(this).val('1');
  });
  jQuery('#children .formrow:last .pagecell:last').html('');
}

function checkLiving(e){
    var whozit = e.name;
    whozit = whozit.replace("[birthdate]", "");
    whozit = whozit.replace("[deathdate]", "");
    var ddate = jQuery('input[name="'+whozit+'[deathdate]"]').val();
    // turn off living flag if death date is entered
    if( ddate.length != 0 ) {
        jQuery('input[name="'+whozit+'[living]"]').attr('checked',false);
        return;
    }

    var bdate = jQuery('input[name="'+whozit+'[birthdate]"]').val();
    if( bdate.length != 0 ) {
        var pattern = /\d{3,4}/g;
        var byear = bdate.match(pattern);
        var thisyear = (new Date).getFullYear();
        // turn off living flag if a hundred years ago or more
        if( thisyear-byear >= 100 ) {
            jQuery('input[name="'+whozit+'[living]"]').attr('checked',false);
            return;
        }
    }
    // turn on living flag
    jQuery('input[name="'+whozit+'[living]"]').attr('checked',true);
}

